﻿/* v:2.0002 Fixed TimeSpan exception in PowerTime [Complete rework check below!] 
* In-game script by MMaster 
* 
* Last Update: 
*  LCDs with no commands will print out message 
*  Damage command argument to modify max damage percentage to show 
*  Fixed Acceleration command exception 
*  Fixed PowerTime command exception 
* 
* MAJOR UPDATE: 
* Complete rework of script core 
*  - automatically updates as many LCDs as possible as fast as possible! 
*  - can be used on ships of any size! 
*  - never throws "Script too complex" exception! 
*  - works optimally with timer block set to Trigger Now 
* 
* New features (check Full Guide for detailed instructions): 
* All commands are now read line by line from Custom Data of LCD panel (LCD Title commands are no longer supported) 
* Text command removed, use TextLCD command to display text from other LCD or combination of Echo, Center and Right commands 
* Ability to add margin on left and right using "!MARGIN:<number of spaces>" (without quotes) in name of LCD 
* Ability to scroll LCDs manually using scrollDown and scrollUp Programmable Block Run arguments 
* "!NOSCROLL" in name of LCD disables automatic scrolling 
* Easy config for monospace fonts (just set the USE_MONOSPACE value below) 
* LCDs now automatically have margin of 1 space (you can override it with "!MARGIN:0" (without quotes)) 
* PowerUsed & PowerUsedTop command to display blocks power input stats 
* Altitude & AltitudeSea command to display altitude above ground & sea 
* Gravity command and its variants to display gravitational acceleration 
* ShipMass & ShipMassBase command to display total/base ship mass 
* StopDistance & StopTime to show estimated distance & time to completely stop ship 
* SpeedKmh & SpeedMph to show speed in km per hour or miles per hour 
* Distance command to show distance of specified block from specified GPS position 
* DamageC command to show only blocks in construction 
* Rework to use new more efficient ways to get power 
* Details command now works even with modded blocks like Nanite Factory by Tyrsis 
* Details command can now start output on line where the user specified text is found 
* TanksX variant to show only percentage without progress bar 
* Tanks and Oxygen commands universal Hydrogen & Oxygen detection for all tanks (even modded ones) 
* Speed and Accel commands progress bar support 
* Added config options for LCDs with exotic texture sizing (below) 
* Moved items definitions higher in the script for easier access 
* Right command which prints text aligned to right 
* Lots of optimizations 
* 
* Disabled for now: 
* Tanks exact values & TanksX, TanksP, TanksV variants (waiting for game support) 
* 
* Previous updates: Look at Change notes tab on Steam workshop page. 
* 
* Customize these: */ 
 
// Use this tag to identify LCDs managed by this script 
// Name filtering rules can be used here so you can use even G:Group or T:[My LCD] 
public const string LCD_TAG = "T:[LCD]"; 
 
// How many lines to scroll per step 
public const int SCROLL_LINES_PER_STEP = 1; 
 
// if you want to use monospace (on all LCDs) change this to true 
public bool USE_MONOSPACE = false; 
 
// Enable initial boot sequence (after compile / world load) 
public const bool ENABLE_BOOT = true; 
 
/* READ THIS FULL GUIDE 
http://steamcommunity.com/sharedfiles/filedetails/?id=407158161 
 
Basic video guide 
Please watch the video guide even if you don't understand my English. You can see how things are done there. 
 
https://youtu.be/vqpPQ_20Xso 
 
 
Read Change Notes (above screenshots) for latest updates and new features. 
I notify about updates on twitter so follow if interested. 
 
Please carefully read the FULL GUIDE before asking questions I had to remove guide from here to add more features :( 
Please DO NOT publish this script or its derivations without my permission! Feel free to use it in blueprints! 
 
Special Thanks 
Keen Software House for awesome Space Engineers game 
Malware for contributing to programmable blocks game code 
Textor and CyberVic for their great script related contributions on Keen forums. 
 
Watch Twitter: https://twitter.com/MattsPlayCorner 
and Facebook: https://www.facebook.com/MattsPlayCorner1080p 
for more crazy stuff from me in the future :) 
*/ 
// Force redraw of panels? (default = false) 
// true - forces redraw of panels 
public const bool FORCE_REDRAW = false; 
 
// (for developer) Enable debug 
public const bool EnableDebug = false; 
 
// (for modded lcds) Affects all LCDs managed by this programmable block 
/* LCD height modifier 
0.5f makes the LCD have only 1/2 the lines of normal LCD 
2.0f makes it fit 2x more lines on LCD */ 
public const float HEIGHT_MOD = 1.0f; 
 
/* line width modifier 
0.5f moves the right edge to 50% of normal LCD width 
2.0f makes it fit 200% more text on line */ 
public const float WIDTH_MOD = 1.0f; 
 
// If you use custom monospace font set this to width of single character 
public const float MONOSPACE_WIDTH = 24f; 
 
List<string> BOOT_FRAMES = new List<string>() { 
/* BOOT FRAMES 
* Each @"<text>" marks single frame, add as many as you want each will be displayed for one second 
* @"" is multiline string so you can write multiple lines */ 
@" 
ВКЛЮЧЕНИЕ СИСТЕМЫ" 
, 
@" 
ПРОВЕРКА СОЕДИНЕНИЯ" 
, 
@" 
ЗАГРУЗКА ПРОГРАММЫ" 
}; 
 
void ItemsConf() 
{ 
// ITEMS AND QUOTAS LIST 
// (subType, mainType, quota, display name, short name) 
// VANILLA ITEMS 
Add("Stone", "Ore", 1000000000, "Камень");  
Add("Iron", "Ore", 1000000000, "Железо");  
Add("Nickel", "Ore", 1000000000, "Никель");  
Add("Cobalt", "Ore", 1000000000, "Кобальт");  
Add("Magnesium", "Ore", 1000000000, "Магний");  
Add("Silicon", "Ore", 1000000000, "Кремний");  
Add("Silver", "Ore", 1000000000, "Серебро");  
Add("Gold", "Ore", 1000000000, "Золото");  
Add("Platinum", "Ore", 1000000000, "Платина");  
Add("Uranium", "Ore", 1000000000, "Уран");  
Add("Ice", "Ore", 1000000000, "Лед");  
Add("Naquadah", "Ore", 1000000000, "Накуада");  
Add("Marble", "Ore", 1000000000, "Мрамор");
Add("Trinium", "Ore", 1000000000, "Триниум");  
Add("Scrap", "Ore", 1000000000, "Металолом");
Add("Copper", "Ore", 1000000000, "Медь");
Add("Zinc", "Ore", 1000000000, "Цинк"); 
Add("Titan", "Ore", 1000000000, "Титан"); 
Add("Neutronium", "Ore", 1000000000, "Нитрониум");

   
Add("Stone", "Ingot", 1000000000, "Гравий");  
Add("Iron", "Ingot", 1000000000, "Железо");  
Add("Nickel", "Ingot", 1000000000, "Никель");  
Add("Cobalt", "Ingot", 1000000000, "Кобальт");  
Add("Magnesium", "Ingot", 1000000000, "Магний");  
Add("Silicon", "Ingot", 1000000000, "Кремний");  
Add("Silver", "Ingot", 1000000000, "Серебро");  
Add("Gold", "Ingot", 1000000000, "Золото");  
Add("Platinum", "Ingot", 1000000000, "Платина");  
Add("Uranium", "Ingot", 1000000000, "Уран");  
Add("Naquadah", "Ingot", 1000000000, "Накуада");
Add("Trinium", "Ingot", 1000000000, "Триниум");  
Add("dcMarblePlateIngot", "Ingot", 1000000000, "Мраморная пластина");  
Add("dcCopperRollIngot", "Ingot", 1000000000, "Медь");   
Add("dcZincIngot", "Ingot", 1000000000, "Цинк");    
Add("dcTitanIngot", "Ingot", 1000000000, "Титан");     
Add("Neutronium", "Ingot", 1000000000, "Нитрониум");   


Add("AutomaticRifleItem", "Tool", 0, "Автомат");  
Add("PreciseAutomaticRifleItem", "Tool", 0, "* Точный Автомат");  
Add("RapidFireAutomaticRifleItem", "Tool", 0, "** Скорострельный Автомат");  
Add("UltimateAutomaticRifleItem", "Tool", 0, "*** Элитный Автомат");  
Add("WelderItem", "Tool", 0, "Автоген");  
Add("Welder2Item", "Tool", 0, "* Улучшенный Автоген");  
Add("Welder3Item", "Tool", 0, "** Профессиональный Автоген");  
Add("Welder4Item", "Tool", 0, "*** Элитный Автоген");  
Add("AngleGrinderItem", "Tool", 0, "Болгарка");  
Add("AngleGrinder2Item", "Tool", 0, "* Улучшенная Болгарка");  
Add("AngleGrinder3Item", "Tool", 0, "** Професииональная Болгарка ");  
Add("AngleGrinder4Item", "Tool", 0, "*** Элитная Болгарка");  
Add("HandDrillItem", "Tool", 0, "Отбойник");  
Add("HandDrill2Item", "Tool", 0, "* Улучшенный Отбойник");  
Add("HandDrill3Item", "Tool", 0, "** Професиональный Отбойник");  
Add("HandDrill4Item", "Tool", 0, "*** Элитный Отбойник");  
Add("Construction", "Component", 5000000, "Стройкомпоненты");  
Add("MetalGrid", "Component", 5000000, "Металлическая решётка");  
Add("InteriorPlate", "Component", 5000000, "Пластина");  
Add("SteelPlate", "Component", 5000000, "Стальна пластина");  
Add("Girder", "Component", 5000000, "Балка");  
Add("SmallTube", "Component", 5000000, "Маленькая труба");  
Add("LargeTube", "Component", 5000000, "Большая труба");  
Add("Motor", "Component", 5000000, "Мотор");  
Add("Display", "Component", 5000000, "Экран");  
Add("BulletproofGlass", "Component", 5000000, "Бронировнное стекло");  
Add("Computer", "Component", 5000000, "Компьютер");  
Add("Reactor", "Component", 5000000, "Компоненты реактора");  
Add("Thrust", "Component", 5000000, "Детали ускорителя");  
Add("GravityGenerator", "Component", 5000000, "Гравикомпоненты", "gravgen");  
Add("Medical", "Component", 5000000, "Медкомпоненты");  
Add("RadioCommunication", "Component", 5000000, "Радиокомпоненты", "radio");  
Add("Detector", "Component", 5000000, "Детекторкомпоненты");  
Add("Explosives", "Component", 5000000, "Взрывчатка");  
Add("SolarCell", "Component", 5000000, "Солнечная батарея"); 
Add("SolarCellBlack", "Component", 5000000, "Чёрная Солнечная батарея"); 
Add("SolarCellGold", "Component", 5000000, "Золотая Солнечная батарея"); 
Add("PowerCell", "Component", 5000000, "Аккумулятор");  
Add("Superconductor", "Component", 5000000, "Сверхпроводник"); 
Add("Zement", "Component", 5000000, "Цемент");
Add("SandGranulat", "Component", 5000000, "Смесь песка и камня"); 
Add("ZPM", "Component", 5000000, "МНТ (Модуль Нулевой Точки)"); 
Add("Canvas", "Component", 5000000, "Парашют");
Add("Trinium", "Component", 5000000, "Плата из Триниума");
Add("Naquadah", "Component", 5000000, "Накуада компонент (кристалл накуады)");
Add("NeutroniumСrate", "Component", 5000000, "Проводник из нитрониума");  
Add("dcBrassPlate", "Component", 5000000, "Латунная пластина"); 
Add("dcCopperPlate", "Component", 5000000, "Медная пластина");  
Add("dcTitanPlate", "Component", 5000000, "Титановая пластина");   
Add("dcZincPlate", "Component", 5000000, "Цинковая пластина");   
Add("dcMarbleBlock", "Component", 5000000, "Блок мрамора");   
Add("DroneWeapon", "Component", 5000000, "Дистанционная бомба-дрон");
Add("LiquidNaquadah", "Component", 5000000, "Энергоблок накуада");   
Add("NickielAllodyPlate", "Component", 5000000, "Никелевая пластина");
  

     
Add("NATO_5p56x45mm", "Ammo", 5000000, "5.56x45mm", "5.56x45mm");  
Add("NATO_25x184mm", "Ammo", 5000000, "25x184mm", "25x184mm");  
Add("Missile200mm", "Ammo", 5000000, "200mm Ракеты", "200mmmissile");  
Add("OxygenBottle", "OxygenContainerObject", 5, "Кислородные Балоны");  
Add("HydrogenBottle", "GasContainerObject", 5, "Водородные Балоны");  
  
// MODDED ITEMS 
// (subType, mainType, quota, display name, short name, used) 
// * if used is true, item will be shown in inventory even for 0 items 
// * if used is false, item will be used only for display name and short name 
// AzimuthSupercharger 
Add("AzimuthSupercharger", "Component", 1600, "Supercharger", "supercharger", false); 
// OKI Ammo 
Add("OKI23mmAmmo", "Ammo", 500, "23x180mm", "23x180mm", false); 
Add("OKI50mmAmmo", "Ammo", 500, "50x450mm", "50x450mm", false); 
Add("OKI122mmAmmo", "Ammo", 200, "122x640mm", "122x640mm", false); 
Add("OKI230mmAmmo", "Ammo", 100, "230x920mm", "230x920mm", false); 
 
// REALLY REALLY REALLY 
// DO NOT MODIFY ANYTHING BELOW THIS (TRANSLATION STRINGS ARE AT THE BOTTOM) 
} 
void Add(string sT, string mT, int q = 0, string dN = "", string sN = "", bool u = true) { MMItems.Add(sT, mT, q, dN, sN, u); } 
 tb 
jd;sS je;t8 jf=null;public Program(){ItemsConf();}void jg(tb a){if(USE_MONOSPACE)tc.sf=MONOSPACE_WIDTH;jf=new t8(this,EnableDebug, 
a);jf.qI=LCD_TAG;jf.qJ=SCROLL_LINES_PER_STEP;jf.qK=ENABLE_BOOT;jf.qL=BOOT_FRAMES;jf.qM=FORCE_REDRAW;jf.qP=HEIGHT_MOD;jf.qO= 
WIDTH_MOD;}void jh(){jd.s0=this;jf.qX=this;}void Main(string b){if(jd==null){jd=new tb(this,EnableDebug);jg(jd);je=new sS(jf);jd. 
s2(je,0);}else{jh();jf.qZ.ph();}if(b!=""){if(je.nn(b))return;}je.n9=0;jd.s5();}class sk:sN {public sk(){rL=7;rH="CmdInvList";} 
float ji=27;float jj=118;public override void Init(){ju=new sZ(rP,mn.qZ);jv=new t2(rP);jj=tc.sh(" / 888.8 M");ji=tc.sg(' ');}void 
jk(string d,double e,int f){if(f>0){mn.rf(Math.Min(100,100*e/f),0.3f);mn.Add(' '+d+' ');mn.rb(sU.nB(e),1.0f,jj);mn.r7(" / "+sU.nB( 
f));}else{mn.Add(d+':');mn.ra(sU.nB(e),1.0f,ji);}}void jl(string g,double i,double j,int k){if(k>0){mn.Add(g+' ');mn.rb(sU.nB(i), 
0.51f);mn.Add(" / "+sU.nB(k));mn.ra("+"+sU.nB(j)+" "+M.T["I1"],1.0f);mn.re(Math.Min(100,100*i/k));}else{mn.Add(g+':');mn.rb(sU.nB( 
i),0.51f);mn.ra("+"+sU.nB(j)+" "+M.T["I1"],1.0f);}}List<t3>jm;int jn=0;int jo=0;bool jp(bool l,bool n,string o,string q){if(!l){jo 
=0;jn=0;}if(jo==0){if((jm=jv.pS(o,l))==null)return false;jo++;l=false;}if(jm.Count>0){if(!n&&!l){if(mn.r4.rr>0&&mn.r4.rq[0]!="")mn 
.r7("");mn.rc("<< "+q+" "+M.T["I2"]+" >>");}for(;jn<jm.Count;jn++){if(!rP.sa(30))return false;double r=jm[jn].p_;if(jz&&r>=jm[jn]. 
pV)continue;int u=jm[jn].pW;if(jz)u=jm[jn].pV;var v=sU.nz(jm[jn].pX,jm[jn].pY);jk(v,r,u);}}return true;}List<t3>jq;int jr=0;int js 
=0;bool jt(bool w){if(!w){jr=0;js=0;}if(js==0){if((jq=jv.pS("Ingot",w))==null)return false;js++;w=false;}if(jq.Count>0){if(!jA&&!w 
){if(mn.r4.rr>0&&mn.r4.rq[0]!="")mn.r7("");mn.rc("<< "+M.T["I4"]+" "+M.T["I2"]+" >>");}for(;jr<jq.Count;jr++){if(!rP.sa(40))return 
false;double x=jq[jr].p_;if(jz&&x>=jq[jr].pV)continue;int y=jq[jr].pW;if(jz)y=jq[jr].pV;var z=sU.nz(jq[jr].pX,jq[jr].pY);if(jq[jr] 
.pX!="Scrap"){double A=jv.pP(jq[jr].pX+" Ore",jq[jr].pX,"Ore").p_;jl(z,x,A,y);}else jk(z,x,y);}}return true;}sZ ju=null;t2 jv;List 
<sX>jw;bool jx,jy,jz,jA;int jB,jC;void jD(){ju.pa();jx=mm.o6.EndsWith("x")||mm.o6.EndsWith("xs");jy=mm.o6.EndsWith("s")||mm.o6. 
EndsWith("sx");jz=mm.o6.StartsWith("missing");jA=mm.o6.StartsWith("invlist");jv.pK();jw=mm.oc;if(jw.Count==0)jw.Add(new sX("all")) 
;}bool jE(bool B){if(!B)jB=0;for(;jB<jw.Count;jB++){sX C=jw[jB];C.om();var D=C.oj.ToLower();if(!B)jC=0;else B=false;for(;jC<C.ol. 
Count;jC++){if(!rP.sa(30))return false;string[]E=C.ol[jC].ToLower().Split(':');double F;if(E[0]=="all")E[0]="";int G=1;int H=-1;if 
(E.Length>1){if(Double.TryParse(E[1],out F)){if(jz)G=(int)Math.Ceiling(F);else H=(int)Math.Ceiling(F);}}var I=E[0];if(D!="")I+=' ' 
+D;jv.pL(I,(C.oi=="-"),G,H);}}return true;}int jF=0;int jG=0;int jH=0;bool jI(bool J){sZ K=ju;if(!J)jF=0;for(;jF<K.pb();jF++){if(! 
J)jG=0;for(;jG<K.oR[jF].GetInventoryCount();jG++){if(!J)jH=0;else J=false;IMyInventory L=K.oR[jF].GetInventory(jG);List< 
IMyInventoryItem>N=L.GetItems();for(;jH<N.Count;jH++){if(!rP.sa(40))return false;IMyInventoryItem O=N[jH];var P=sU.nx(O);var Q=P. 
ToLower();string R,S;sU.ny(Q,out R,out S);if(S=="ore"){if(jv.pN(R.ToLower()+" ingot",R,"Ingot")&&jv.pN(P,R,S))continue;}else{if(jv 
.pN(P,R,S))continue;}sU.ny(P,out R,out S);t3 U=jv.pP(Q,R,S);U.p_+=(double)O.Amount;}}}return true;}int jJ=0;public override bool 
RunCmd(bool V){if(!V){jD();jJ=0;}for(;jJ<=9;jJ++){switch(jJ){case 0:if(!ju.p0(mm.o7,V))return false;break;case 1:if(!jE(V))return 
false;if(!jx){if(!jv.pU(V))return false;}break;case 2:if(!jI(V))return false;break;case 3:if(!jp(V,jA,"Ore",M.T["I3"]))return 
false;break;case 4:if(jy){if(!jp(V,jA,"Ingot",M.T["I4"]))return false;}else{if(!jt(V))return false;}break;case 5:if(!jp(V,jA, 
"Component",M.T["I5"]))return false;break;case 6:if(!jp(V,jA,"OxygenContainerObject",M.T["I6"]))return false;break;case 7:if(!jp(V 
,true,"GasContainerObject",""))return false;break;case 8:if(!jp(V,jA,"AmmoMagazine",M.T["I7"]))return false;break;case 9:if(!jp(V, 
jA,"PhysicalGunObject",M.T["I8"]))return false;break;}V=false;}return true;}}class sl:sN {sZ jK;public sl(){rL=2;rH="CmdCargo";} 
public override void Init(){jK=new sZ(rP,mn.qZ);}bool jL=true;bool jM=false;bool jN=false;double jO=0;double jP=0;int jQ=0;public 
override bool RunCmd(bool W){if(!W){jK.pa();jL=mm.o6.Contains("all");jM=(mm.o6[mm.o6.Length-1]=='x');jN=(mm.o6[mm.o6.Length-1]== 
'p');jO=0;jP=0;jQ=0;}if(jQ==0){if(jL){if(!jK.p0(mm.o7,W))return false;}else{if(!jK.p8("cargocontainer",mm.o7,W))return false;}jQ++ 
;W=false;}double X=jK.oT(ref jO,ref jP,W);if(Double.IsNaN(X))return false;mn.Add(M.T["C2"]+" ");if(!jM&&!jN){mn.ra(sU.nB(jO)+ 
"L / "+sU.nB(jP)+"L");mn.rf(X,1.0f,mn.qQ);mn.r7(' '+sU.nC(X)+"%");}else if(jN){mn.ra(sU.nC(X)+"%");mn.re(X);}else mn.ra(sU.nC(X)+ 
"%");return true;}}class sm:sN {sZ jR;public sm(){rL=2;rH="CmdMass";}public override void Init(){jR=new sZ(rP,mn.qZ);}bool jS= 
false;bool jT=false;int jU=0;public override bool RunCmd(bool Y){if(!Y){jR.pa();jS=(mm.o6[mm.o6.Length-1]=='x');jT=(mm.o6[mm.o6. 
Length-1]=='p');jU=0;}if(jU==0){if(!jR.p0(mm.o7,Y))return false;jU++;Y=false;}double Z=jR.oW(Y);if(Double.IsNaN(Z))return false; 
double _=0;int a0=mm.oc.Count;if(a0>0){double.TryParse(mm.oc[0].ok.Trim(),out _);if(a0>1){var a1=mm.oc[1].ok.Trim().ToLower();if( 
a1!="")_*=Math.Pow(1000.0,"kmgtpezy".IndexOf(a1[0]));}_*=1000.0;}mn.Add(M.T["M1"]+" ");if(_<=0){mn.ra(sU.nB(Z));return true;} 
double a2=Z/_*100;if(!jS&&!jT){mn.ra(sU.nB(Z)+"g / "+sU.nB(_)+"g");mn.rf(a2,1.0f,mn.qQ);mn.r7(' '+sU.nC(a2)+"%");}else if(jT){mn. 
ra(sU.nC(a2)+"%");mn.re(a2);}else mn.ra(sU.nC(a2)+"%");return true;}}class sn:sN {t5 jV;sZ jW;public sn(){rL=3;rH="CmdOxygen";} 
public override void Init(){jV=mn.qY;jW=new sZ(rP,mn.qZ);}int jX=0;int jY=0;bool jZ=false;int j_=0;double k0=0;double k1=0;double 
k2;public override bool RunCmd(bool a3){if(!a3){jW.pa();jX=0;jY=0;k2=0;}if(jX==0){if(!jW.p8("airvent",mm.o7,a3))return false;jZ=( 
jW.pb()>0);jX++;a3=false;}if(jX==1){for(;jY<jW.pb();jY++){if(!rP.sa(8))return false;IMyAirVent a4=jW.oR[jY]as IMyAirVent;k2=Math. 
Max(a4.GetOxygenLevel()*100,0f);mn.Add(a4.CustomName);if(a4.CanPressurize)mn.ra(sU.nC(k2)+"%");else mn.ra(M.T["O1"]);mn.re(k2);}jX 
++;a3=false;}if(jX==2){if(!a3)jW.pa();if(!jW.p8("oxyfarm",mm.o7,a3))return false;j_=jW.pb();jX++;a3=false;}if(jX==3){if(j_>0){if(! 
a3)jY=0;double a5=0;for(;jY<j_;jY++){if(!rP.sa(4))return false;IMyOxygenFarm a6=jW.oR[jY]as IMyOxygenFarm;a5+=a6.GetOutput()*100;} 
k2=a5/j_;if(jZ)mn.r7("");jZ|=(j_>0);mn.Add(M.T["O2"]);mn.ra(sU.nC(k2)+"%");mn.re(k2);}jX++;a3=false;}if(jX==4){if(!a3)jW.pa();if(! 
jW.p8("oxytank",mm.o7,a3))return false;j_=jW.pb();if(j_==0){if(!jZ)mn.r7(M.T["O3"]);return true;}jX++;a3=false;}if(jX==5){if(!a3){ 
k0=0;k1=0;jY=0;}if(!jV.qh(jW.oR,"oxygen",ref k1,ref k0,a3))return false;if(k0==0){if(!jZ)mn.r7(M.T["O3"]);return true;}k2=k1/k0* 
100;if(jZ)mn.r7("");mn.Add(M.T["O4"]);mn.ra(sU.nC(k2)+"%");mn.re(k2);jX++;}return true;}}class so:sN {t5 k3;sZ k4;public so(){rL=2 
;rH="CmdTanks";}public override void Init(){k3=mn.qY;k4=new sZ(rP,mn.qZ);}int k5=0;string k6;string k7;double k8=0;double k9=0; 
double ka;public override bool RunCmd(bool a7){List<sX>a8=mm.oc;if(a8.Count==0){mn.r7(M.T["T4"]);return true;}if(!a7){k6=(mm.o6. 
EndsWith("x")?"s":(mm.o6.EndsWith("p")?"p":(mm.o6.EndsWith("v")?"v":"n")));k5=0;k7=a8[0].ok.Trim().ToLower();k4.pa();k8=0;k9=0;}if 
(k5==0){if(!k4.p8("oxytank",mm.o7,a7))return false;a7=false;k5++;}if(k5==1){if(!k3.qh(k4.oR,k7,ref k8,ref k9,a7))return false;a7= 
false;k5++;}if(k9==0){mn.r7(String.Format(M.T["T5"],k7));return true;}ka=k8/k9*100;k7=char.ToUpper(k7[0])+k7.Substring(1);mn.Add( 
k7+" "+M.T["T6"]);switch(k6){case "s":mn.ra(' '+sU.nC(ka)+"%");break;default:mn.ra(' '+sU.nC(ka)+"%");mn.re(ka);break;}return true 
;}}class sp:sN {public sp(){rL=7;rH="CmdPowerTime";}class sq {public TimeSpan kA=new TimeSpan(-1);public double kB=-1;public 
double kC=0;}sq kb=new sq();sZ kc;sZ kd;public override void Init(){kc=new sZ(rP,mn.qZ);kd=new sZ(rP,mn.qZ);}int ke=0;double kf=0; 
double kg=0,kh=0;double ki=0,kj=0,kk=0;double kl=0,km=0;int kn=0;bool ko(string a9,out TimeSpan aa,out double ab,bool ac){ 
MyResourceSourceComponent ad;MyResourceSinkComponent ae;double af=rK;sq ag=kb;aa=ag.kA;ab=ag.kB;if(!ac){kc.pa();kd.pa();ag.kB=0;ke 
=0;kf=0;kg=kh=0;ki=0;kj=kk=0;kl=km=0;kn=0;}if(ke==0){if(!kc.p8("reactor",a9,ac))return false;ac=false;ke++;}if(ke==1){for(;kn<kc. 
oR.Count;kn++){if(!rP.sa(6))return false;IMyReactor ah=kc.oR[kn]as IMyReactor;if(ah==null||!ah.IsWorking)continue;if(ah.Components 
.TryGet<MyResourceSourceComponent>(out ad)){kg+=ad.CurrentOutputByType(mn.qY.q9);kh+=ad.MaxOutputByType(mn.qY.q9);}kf+=(double)ah. 
GetInventory(0).CurrentMass;}ac=false;ke++;}if(ke==2){if(!kd.p8("battery",a9,ac))return false;ac=false;ke++;}if(ke==3){if(!ac)kn=0 
;for(;kn<kd.oR.Count;kn++){if(!rP.sa(15))return false;IMyBatteryBlock ah=kd.oR[kn]as IMyBatteryBlock;if(ah==null||!ah.IsWorking) 
continue;if(ah.Components.TryGet<MyResourceSourceComponent>(out ad)){kj=ad.CurrentOutputByType(mn.qY.q9);kk=ad.MaxOutputByType(mn. 
qY.q9);}if(ah.Components.TryGet<MyResourceSinkComponent>(out ae)){kj-=ae.CurrentInputByType(mn.qY.q9);}double aj=(kj<0?(ah. 
MaxStoredPower-ah.CurrentStoredPower)/(-kj/3600):0);if(aj>ag.kB)ag.kB=aj;if(ah.GetValueBool("Recharge"))continue;kl+=kj;km+=kk;ki 
+=ah.CurrentStoredPower;}ac=false;ke++;}double ak=kg+kl;if(ak<=0)ag.kA=TimeSpan.FromSeconds(-1);else{double al=ag.kA.TotalSeconds; 
double am;double an=(ag.kC-kf)/af;if(kg<=0)an=Math.Min(ak,kh)/3600000;double ao=0;if(km>0)ao=Math.Min(ak,km)/3600;if(an<=0&&ao<=0) 
am=-1;else if(an<=0)am=ki/ao;else if(ao<=0)am=kf/an;else{double ap=ao;double aq=(kg<=0?ak/3600:an*ak/kg);am=ki/ap+kf/aq;}if(al<=0 
||am<0)al=am;else al=(al+am)/2;try{ag.kA=TimeSpan.FromSeconds(al);}catch{ag.kA=TimeSpan.FromSeconds(-1);}}ag.kC=kf;ab=ag.kB;aa=ag. 
kA;return true;}string kp(TimeSpan at){var au="";if(at.Ticks<=0)return "-";if((int)at.TotalDays>0)au+=(long)at.TotalDays+" "+M.T[ 
"C5"]+" ";if(at.Hours>0||au!="")au+=at.Hours+"h ";if(at.Minutes>0||au!="")au+=at.Minutes+"m ";return au+at.Seconds+"s";}int kq=0; 
bool kr=false;bool ks=false;double kt=0;TimeSpan ku;int kv=0,kw=0,kx=0;int ky=0;int kz=0;public override bool RunCmd(bool av){if(! 
av){kr=(mm.o6[mm.o6.Length-1]=='x');ks=(mm.o6[mm.o6.Length-1]=='p');kq=0;kv=kw=kx=ky=0;kz=0;kt=0;}if(kq==0){if(mm.oc.Count>0){for( 
;kz<mm.oc.Count;kz++){if(!rP.sa(100))return false;mm.oc[kz].om();if(mm.oc[kz].ol.Count<=0)continue;var aw=mm.oc[kz].ol[0];int. 
TryParse(aw,out ky);if(kz==0)kv=ky;else if(kz==1)kw=ky;else if(kz==2)kx=ky;}}kq++;av=false;}if(kq==1){if(!ko(mm.o7,out ku,out kt, 
av))return false;kq++;av=false;}if(!rP.sa(30))return false;double ax=0;TimeSpan ay;try{ay=new TimeSpan(kv,kw,kx);}catch{ay= 
TimeSpan.FromSeconds(-1);}string az;if(ku.TotalSeconds>0||kt<=0){mn.Add(M.T["PT1"]+" ");az=kp(ku);ax=ku.TotalSeconds;}else{mn.Add( 
M.T["PT2"]+" ");az=kp(TimeSpan.FromSeconds(kt));if(ay.TotalSeconds>=kt)ax=ay.TotalSeconds-kt;else ax=0;}if(ay.Ticks<=0){mn.ra(az); 
return true;}double aA=ax/ay.TotalSeconds*100;if(aA>100)aA=100;if(!kr&&!ks){mn.ra(az);mn.rf(aA,1.0f,mn.qQ);mn.r7(' '+aA.ToString( 
"0.0")+"%");}else if(ks){mn.ra(aA.ToString("0.0")+"%");mn.re(aA);}else mn.ra(aA.ToString("0.0")+"%");return true;}}class sr:sN { 
public sr(){rL=7;rH="CmdPowerUsed";}t5 kD;sZ kE;public override void Init(){kE=new sZ(rP,mn.qZ);kD=mn.qY;}string kF;string kG; 
string kH;void kI(double aB,double aC){double aD=(aC>0?aB/aC*100:0);switch(kF){case "s":mn.ra(aD.ToString("0.0")+"%",1.0f);break; 
case "v":mn.ra(sU.nB(aB)+"W / "+sU.nB(aC)+"W",1.0f);break;case "c":mn.ra(sU.nB(aB)+"W",1.0f);break;case "p":mn.ra(aD.ToString( 
"0.0")+"%",1.0f);mn.re(aD);break;default:mn.ra(sU.nB(aB)+"W / "+sU.nB(aC)+"W");mn.rf(aD,1.0f,mn.qQ);mn.ra(' '+aD.ToString("0.0")+ 
"%");break;}}double kJ=0,kK=0;int kL=0;int kM=0;ss kN=new ss();public override bool RunCmd(bool aE){if(!aE){kF=(mm.o6.EndsWith("x" 
)?"s":(mm.o6.EndsWith("usedp")||mm.o6.EndsWith("topp")?"p":(mm.o6.EndsWith("v")?"v":(mm.o6.EndsWith("c")?"c":"n"))));kG=(mm.o6. 
Contains("top")?"top":"");kH=(mm.oc.Count>0?mm.oc[0].ok:M.T["PU1"]);kJ=kK=0;kM=0;kL=0;kE.pa();kN.kS();}if(kM==0){if(!kE.p0(mm.o7, 
aE))return false;aE=false;kM++;}MyResourceSinkComponent aF;MyResourceSourceComponent aG;switch(kG){case "top":if(kM==1){for(;kL<kE 
.oR.Count;kL++){if(!rP.sa(20))return false;IMyTerminalBlock aH=kE.oR[kL];if(aH.Components.TryGet<MyResourceSinkComponent>(out aF)) 
{ListReader<MyDefinitionId>aI=aF.AcceptedResources;if(aI.IndexOf(kD.q9)<0)continue;kJ=aF.CurrentInputByType(kD.q9)*0xF4240;}else  
continue;kN.kP(kJ,aH);}aE=false;kM++;}if(kN.kQ()<=0){mn.r7("PowerUsedTop: "+M.T["D2"]);return true;}int aJ=10;if(mm.oc.Count>0)if( 
!int.TryParse(kH,out aJ)){aJ=10;}if(aJ>kN.kQ())aJ=kN.kQ();if(kM==2){if(!aE){kL=kN.kQ()-1;kN.kT();}for(;kL>=kN.kQ()-aJ;kL--){if(!rP 
.sa(30))return false;IMyTerminalBlock aH=kN.kR(kL);var aK=tc.si(aH.CustomName,t8.qG*0.4f*mn.r4.rl);if(aH.Components.TryGet< 
MyResourceSinkComponent>(out aF)){kJ=aF.CurrentInputByType(kD.q9)*0xF4240;kK=aF.MaxRequiredInputByType(kD.q9)*0xF4240;}mn.Add(aK+ 
" ");kI(kJ,kK);}}break;default:for(;kL<kE.oR.Count;kL++){if(!rP.sa(10))return false;double aL;IMyTerminalBlock aH=kE.oR[kL];if(aH. 
Components.TryGet<MyResourceSinkComponent>(out aF)){ListReader<MyDefinitionId>aI=aF.AcceptedResources;if(aI.IndexOf(kD.q9)<0) 
continue;aL=aF.CurrentInputByType(kD.q9);kK+=aF.MaxRequiredInputByType(kD.q9);}else continue;if(aH.Components.TryGet< 
MyResourceSourceComponent>(out aG)&&(aH as IMyBatteryBlock!=null)){aL-=aG.CurrentOutputByType(kD.q9);if(aL<=0)continue;}kJ+=aL;}mn 
.Add(kH);kI(kJ*0xF4240,kK*0xF4240);break;}return true;}public class ss {List<KeyValuePair<double,IMyTerminalBlock>>kO=new List< 
KeyValuePair<double,IMyTerminalBlock>>();public void kP(double aP,IMyTerminalBlock aQ){kO.Add(new KeyValuePair<double, 
IMyTerminalBlock>(aP,aQ));}public int kQ(){return kO.Count;}public IMyTerminalBlock kR(int aR){return kO[aR].Value;}public void kS 
(){kO.Clear();}public void kT(){kO.Sort((aS,aT)=>(aS.Key.CompareTo(aT.Key)));}}}class st:sN {public st(){rL=3;rH="CmdPower";}t5 kU 
;sZ kV;sZ kW;sZ kX;sZ kY;public override void Init(){kV=new sZ(rP,mn.qZ);kW=new sZ(rP,mn.qZ);kX=new sZ(rP,mn.qZ);kY=new sZ(rP,mn. 
qZ);kU=mn.qY;}string kZ;bool k_;string l0;string l1;int l2;int l3=0;public override bool RunCmd(bool aU){if(!aU){kZ=(mm.o6. 
EndsWith("x")?"s":(mm.o6.EndsWith("p")?"p":(mm.o6.EndsWith("v")?"v":"n")));k_=(mm.o6.StartsWith("powersummary"));l0="a";l1="";if( 
mm.o6.Contains("stored"))l0="s";else if(mm.o6.Contains("in"))l0="i";else if(mm.o6.Contains("out"))l0="o";l3=0;kV.pa();kW.pa();kX. 
pa();}if(l0=="a"){if(l3==0){if(!kV.p8("reactor",mm.o7,aU))return false;aU=false;l3++;}if(l3==1){if(!kW.p8("solarpanel",mm.o7,aU)) 
return false;aU=false;l3++;}}else if(l3==0)l3=2;if(l3==2){if(!kX.p8("battery",mm.o7,aU))return false;aU=false;l3++;}int aV=kV.pb() 
;int aW=kW.pb();int aX=kX.pb();if(l3==3){l2=0;if(aV>0)l2++;if(aW>0)l2++;if(aX>0)l2++;if(l2<1){mn.r7(M.T["P6"]);return true;}if(mm. 
oc.Count>0){if(mm.oc[0].ok.Length>0)l1=mm.oc[0].ok;}l3++;aU=false;}if(l0!="a"){if(!li(kX,(l1==""?M.T["P7"]:l1),l0,kZ,aU))return 
false;return true;}var aY=M.T["P8"];if(!k_){if(l3==4){if(aV>0)if(!l9(kV,(l1==""?M.T["P9"]:l1),kZ,aU))return false;l3++;aU=false;} 
if(l3==5){if(aW>0)if(!l9(kW,(l1==""?M.T["P10"]:l1),kZ,aU))return false;l3++;aU=false;}if(l3==6){if(aX>0)if(!li(kX,(l1==""?M.T["P7" 
]:l1),l0,kZ,aU))return false;l3++;aU=false;}}else{aY=M.T["P11"];l2=10;if(l3==4)l3=7;}if(l2==1)return true;if(!aU){kY.pa();kY.p9(kV 
);kY.p9(kW);kY.p9(kX);}if(!l9(kY,aY,kZ,aU))return false;return true;}void l4(double aZ,double a_){double b0=(a_>0?aZ/a_*100:0); 
switch(kZ){case "s":mn.ra(' '+b0.ToString("0.0")+"%");break;case "v":mn.ra(sU.nB(aZ)+"W / "+sU.nB(a_)+"W");break;case "c":mn.ra(sU 
.nB(aZ)+"W");break;case "p":mn.ra(' '+b0.ToString("0.0")+"%");mn.re(b0);break;default:mn.ra(sU.nB(aZ)+"W / "+sU.nB(a_)+"W");mn.rf( 
b0,1.0f,mn.qQ);mn.ra(' '+b0.ToString("0.0")+"%");break;}}double l5=0;double l6=0,l7=0;int l8=0;bool l9(sZ b1,string b2,string b3, 
bool b4){if(!b4){l6=0;l7=0;l8=0;}if(l8==0){if(!kU.qf(b1.oR,kU.q9,ref l5,ref l5,ref l6,ref l7,b4))return false;l8++;b4=false;}if(! 
rP.sa(50))return false;double b5=(l7>0?l6/l7*100:0);mn.Add(b2+": ");l4(l6*0xF4240,l7*0xF4240);return true;}double la=0,lb=0,lc=0, 
ld=0;double le=0,lg=0;int lh=0;bool li(sZ b6,string b7,string b8,string b9,bool ba){if(!ba){la=lb=0;lc=ld=0;le=lg=0;lh=0;}if(lh==0 
){if(!kU.qd(b6.oR,ref lc,ref ld,ref la,ref lb,ref le,ref lg,ba))return false;lc*=0xF4240;ld*=0xF4240;la*=0xF4240;lb*=0xF4240;le*= 
0xF4240;lg*=0xF4240;lh++;ba=false;}double bb=(lg>0?le/lg*100:0);double bc=(lb>0?la/lb*100:0);double bd=(ld>0?lc/ld*100:0);var be= 
b8=="a";if(lh==1){if(!rP.sa(50))return false;if(be){if(b9!="p"){mn.Add(b7+": ");mn.ra("(IN "+sU.nB(lc)+"W / OUT "+sU.nB(la)+"W)"); 
}else mn.r7(b7+": ");mn.Add("  "+M.T["P3"]+": ");}else mn.Add(b7+": ");if(be||b8=="s")switch(b9){case "s":mn.ra(' '+bb.ToString( 
"0.0")+"%");break;case "v":mn.ra(sU.nB(le)+"Wh / "+sU.nB(lg)+"Wh");break;case "p":mn.ra(' '+bb.ToString("0.0")+"%");mn.re(bb); 
break;default:mn.ra(sU.nB(le)+"Wh / "+sU.nB(lg)+"Wh");mn.rf(bb,1.0f,mn.qQ);mn.ra(' '+bb.ToString("0.0")+"%");break;}if(b8=="s") 
return true;lh++;ba=false;}if(lh==2){if(!rP.sa(50))return false;if(be)mn.Add("  "+M.T["P4"]+": ");if(be||b8=="o")switch(b9){case 
"s":mn.ra(' '+bc.ToString("0.0")+"%");break;case "v":mn.ra(sU.nB(la)+"W / "+sU.nB(lb)+"W");break;case "p":mn.ra(' '+bc.ToString( 
"0.0")+"%");mn.re(bc);break;default:mn.ra(sU.nB(la)+"W / "+sU.nB(lb)+"W");mn.rf(bc,1.0f,mn.qQ);mn.ra(' '+bc.ToString("0.0")+"%"); 
break;}if(b8=="o")return true;lh++;ba=false;}if(!rP.sa(50))return false;if(be)mn.Add("  "+M.T["P5"]+": ");if(be||b8=="i")switch(b9 
){case "s":mn.ra(' '+bd.ToString("0.0")+"%");break;case "v":mn.ra(sU.nB(lc)+"W / "+sU.nB(ld)+"W");break;case "p":mn.ra(' '+bd. 
ToString("0.0")+"%");mn.re(bd);break;default:mn.ra(sU.nB(lc)+"W / "+sU.nB(ld)+"W");mn.rf(bd,1.0f,mn.qQ);mn.ra(' '+bd.ToString( 
"0.0")+"%");break;}return true;}}class su:sN {public su(){rL=0.5;rH="CmdSpeed";}public override bool RunCmd(bool bf){double bg=0; 
double bh=1;var bi="m/s";if(mm.o6.Contains("kmh")){bh=3.6;bi="km/h";}else if(mm.o6.Contains("mph")){bh=2.23694;bi="mph";}if(mm.o7 
!="")double.TryParse(mm.o7.Trim(),out bg);mn.Add(M.T["S1"]+" ");mn.ra((mn.q_.os*bh).ToString("F1")+" "+bi+" ");if(bg>0)mn.re(mn.q_ 
.os/bg*100);return true;}}class sv:sN {public sv(){rL=0.5;rH="CmdAccel";}public override bool RunCmd(bool bj){double bk=0;if(mm.o7 
!="")double.TryParse(mm.o7.Trim(),out bk);mn.Add(M.T["AC1"]+" ");mn.ra(mn.q_.ou.ToString("F1")+" m/s²");if(bk>0){double bl=mn.q_. 
ou/bk*100;mn.re(bl);}return true;}}class sw:sN {public sw(){rL=30;rH="CmdEcho";}public override bool RunCmd(bool bm){var bn=(mm.o6 
=="center"?"c":(mm.o6=="right"?"r":"n"));switch(bn){case "c":mn.rc(mm.o9);break;case "r":mn.ra(mm.o9);break;default:mn.r7(mm.o9); 
break;}return true;}}class sx:sN {public sx(){rL=3;rH="CmdCharge";}sZ lj;public override void Init(){lj=new sZ(rP,mn.qZ);}int lk=0 
;int ll=0;public override bool RunCmd(bool bo){var bp=mm.o6.Contains("x");if(!bo){lj.pa();ll=0;lk=0;}if(lk==0){if(!lj.p8( 
"jumpdrive",mm.o7,bo))return false;if(lj.pb()<=0){mn.r7("Charge: "+M.T["D2"]);return true;}lk++;bo=false;}for(;ll<lj.pb();ll++){if 
(!rP.sa(25))return false;IMyJumpDrive bq=lj.oR[ll]as IMyJumpDrive;double br,bs,bt;bt=t0.pp(bq,out br,out bs);mn.Add(bq.CustomName) 
;if(!bp){mn.ra(sU.nB(br)+"Wh / "+sU.nB(bs)+"Wh");mn.rf(bt,1.0f,mn.qQ);}mn.ra(' '+bt.ToString("0.0")+"%");}return true;}}class sy: 
sN {public sy(){rL=1;rH="CmdDateTime";}public override bool RunCmd(bool bu){var bv=(mm.o6.StartsWith("datetime"));var bw=(mm.o6. 
StartsWith("date"));var bx=mm.o6.Contains("c");int by=mm.o6.IndexOf('+');if(by<0)by=mm.o6.IndexOf('-');float bz=0;if(by>=0)float. 
TryParse(mm.o6.Substring(by),out bz);DateTime bA=DateTime.Now.AddHours(bz);var bB="";int bC=mm.o8.IndexOf(' ');if(bC>=0)bB=mm.o8. 
Substring(bC+1);if(!bv){if(!bw)bB+=bA.ToShortTimeString();else bB+=bA.ToShortDateString();}else{if(bB=="")bB=String.Format( 
"{0:d} {0:t}",bA);else{bB=bB.Replace("/","\\/");bB=bB.Replace(":","\\:");bB=bB.Replace("\"","\\\"");bB=bB.Replace("'","\\'");bB=bA 
.ToString(bB+' ');bB=bB.Substring(0,bB.Length-1);}}if(bx)mn.rc(bB);else mn.r7(bB);return true;}}class sz:sN {public sz(){rL=1;rH= 
"CmdCountdown";}public override bool RunCmd(bool bD){var bE=mm.o6.EndsWith("c");var bF=mm.o6.EndsWith("r");var bG="";int bH=mm.o8. 
IndexOf(' ');if(bH>=0)bG=mm.o8.Substring(bH+1).Trim();DateTime bI=DateTime.Now;DateTime bJ;if(!DateTime.TryParseExact(bG, 
"H:mm d.M.yyyy",System.Globalization.CultureInfo.InvariantCulture,System.Globalization.DateTimeStyles.None,out bJ)){mn.r7(M.T["C3" 
]);mn.r7("  Countdown 19:02 28.2.2015");return true;}TimeSpan bK=bJ-bI;var bL="";if(bK.Ticks<=0)bL=M.T["C4"];else{if((int)bK. 
TotalDays>0)bL+=(int)bK.TotalDays+" "+M.T["C5"]+" ";if(bK.Hours>0||bL!="")bL+=bK.Hours+"h ";if(bK.Minutes>0||bL!="")bL+=bK.Minutes 
+"m ";bL+=bK.Seconds+"s";}if(bE)mn.rc(bL);else if(bF)mn.ra(bL);else mn.r7(bL);return true;}}class sA:sN {public sA(){rL=1;rH= 
"CmdTextLCD";}public override bool RunCmd(bool bM){IMyTextPanel bN=mo.mF.ql;if(bN==null)return true;var bO="";if(mm.o7!=""&&mm.o7 
!="*"){IMyTextPanel bP=mn.r0.GetBlockWithName(mm.o7)as IMyTextPanel;if(bP==null){mn.r7("TextLCD: "+M.T["T1"]+mm.o7);return true;} 
bO=bP.GetPublicText();}else{mn.r7("TextLCD:"+M.T["T2"]);return true;}if(bO.Length==0)return true;mn.r8(bO);return true;}}class sB: 
sN {public sB(){rL=15;rH="CmdBlockCount";}sZ lm;public override void Init(){lm=new sZ(rP,mn.qZ);}bool ln;bool lo;int lp=0;int lq=0 
;public override bool RunCmd(bool bQ){if(!bQ){ln=(mm.o6=="enabledcount");lo=(mm.o6=="prodcount");lp=0;lq=0;}if(mm.oc.Count==0){if( 
lq==0){if(!bQ)lm.pa();if(!lm.p0(mm.o7,bQ))return false;lq++;bQ=false;}if(!lA(lm,"blocks",ln,lo,bQ))return false;return true;}for(; 
lp<mm.oc.Count;lp++){sX bR=mm.oc[lp];if(!bQ)bR.om();if(!lt(bR,bQ))return false;bQ=false;}return true;}int lr=0;int ls=0;bool lt(sX 
bS,bool bT){if(!bT){lr=0;ls=0;}for(;lr<bS.ol.Count;lr++){if(ls==0){if(!bT)lm.pa();if(!lm.p8(bS.ol[lr],mm.o7,bT))return false;ls++; 
bT=false;}if(!lA(lm,bS.ol[lr],ln,lo,bT))return false;ls=0;bT=false;}return true;}Dictionary<string,int>lu=new Dictionary<string, 
int>();Dictionary<string,int>lv=new Dictionary<string,int>();List<string>lw=new List<string>();int lx=0;int ly=0;int lz=0;bool lA( 
sZ bU,string bV,bool bW,bool bX,bool bY){string bZ;if(bU.pb()==0){bZ=bV.ToLower();bZ=char.ToUpper(bZ[0])+bZ.Substring(1).ToLower() 
;mn.Add(bZ+" "+M.T["C1"]+" ");var b_=(bW||bX?"0 / 0":"0");mn.ra(b_);return true;}if(!bY){lu.Clear();lv.Clear();lw.Clear();lx=0;ly= 
0;lz=0;}if(lz==0){for(;lx<bU.pb();lx++){if(!rP.sa(15))return false;IMyProductionBlock c0=bU.oR[lx]as IMyProductionBlock;bZ=bU.oR[ 
lx].DefinitionDisplayNameText;if(lw.Contains(bZ)){lu[bZ]++;if((bW&&bU.oR[lx].IsWorking)||(bX&&c0!=null&&c0.IsProducing))lv[bZ]++;} 
else{lu.Add(bZ,1);lw.Add(bZ);if(bW||bX)if((bW&&bU.oR[lx].IsWorking)||(bX&&c0!=null&&c0.IsProducing))lv.Add(bZ,1);else lv.Add(bZ,0) 
;}}lz++;bY=false;}for(;ly<lu.Count;ly++){if(!rP.sa(8))return false;mn.Add(lw[ly]+" "+M.T["C1"]+" ");var b_=(bW||bX?lv[lw[ly]]+ 
" / ":"")+lu[lw[ly]];mn.ra(b_);}return true;}}class sC:sN {public sC(){rL=5;rH="CmdShipCtrl";}sZ lB;public override void Init(){lB 
=new sZ(rP,mn.qZ);}public override bool RunCmd(bool c2){if(!c2)lB.pa();if(!lB.p8("shipctrl",mm.o7,c2))return false;if(lB.pb()<=0){ 
if(mm.o7!=""&&mm.o7!="*")mn.r7(mm.o6+": "+M.T["SC1"]+" ("+mm.o7+")");else mn.r7(mm.o6+": "+M.T["SC1"]);return true;}if(mm.o6. 
StartsWith("damp")){var c3=(lB.oR[0]as IMyShipController).DampenersOverride;mn.Add(M.T["SCD"]);mn.ra(c3?"ON":"OFF");}else{var c3=( 
lB.oR[0]as IMyShipController).IsUnderControl;mn.Add(M.T["SCO"]);mn.ra(c3?"YES":"NO");}return true;}}class sD:sN {public sD(){rL=5; 
rH="CmdWorking";}sZ lC;public override void Init(){lC=new sZ(rP,mn.qZ);}int lD=0;int lE=0;bool lF;public override bool RunCmd(bool 
c5){if(!c5){lD=0;lF=(mm.o6=="workingx");lE=0;}if(mm.oc.Count==0){if(lD==0){if(!c5)lC.pa();if(!lC.p0(mm.o7,c5))return false;lD++;c5 
=false;}if(!lO(lC,lF,"",c5))return false;return true;}for(;lE<mm.oc.Count;lE++){sX c6=mm.oc[lE];if(!c5)c6.om();if(!lL(c6,c5)) 
return false;c5=false;}return true;}int lG=0;int lH=0;string[]lI;string lJ;string lK;bool lL(sX c7,bool c8){if(!c8){lG=0;lH=0;}for 
(;lH<c7.ol.Count;lH++){if(lG==0){if(!c8){if(c7.ol[lH]=="")continue;lC.pa();lI=c7.ol[lH].ToLower().Split(':');lJ=lI[0];lK=(lI. 
Length>1?lI[1]:"");}if(lJ!=""){if(!lC.p8(lJ,mm.o7,c8))return false;}else{if(!lC.p0(mm.o7,c8))return false;}lG++;c8=false;}if(!lO( 
lC,lF,lK,c8))return false;lG=0;c8=false;}return true;}string lM(IMyTerminalBlock c9){if(!c9.IsWorking)return M.T["W1"]; 
IMyProductionBlock ca=c9 as IMyProductionBlock;if(ca!=null)if(ca.IsProducing)return M.T["W2"];else return M.T["W3"];IMyAirVent cb= 
c9 as IMyAirVent;if(cb!=null){if(cb.CanPressurize)return(cb.GetOxygenLevel()*100).ToString("F1")+"%";else return M.T["W4"];} 
IMyOxygenTank cc=c9 as IMyOxygenTank;if(cc!=null)return(cc.GetOxygenLevel()*100).ToString("F1")+"%";IMyBatteryBlock cd=c9 as 
IMyBatteryBlock;if(cd!=null)return t0.pn(cd);IMyJumpDrive ce=c9 as IMyJumpDrive;if(ce!=null)return t0.pq(ce).ToString("0.0")+"%"; 
IMyLandingGear cf=c9 as IMyLandingGear;if(cf!=null)return t0.pr(cf);IMyDoor cg=c9 as IMyDoor;if(cg!=null){if(cg.Open)return M.T[ 
"W5"];return M.T["W6"];}IMyShipConnector ch=c9 as IMyShipConnector;if(ch!=null)if(ch.IsLocked){if(ch.IsConnected)return M.T["W7"]; 
else return M.T["W10"];}else return M.T["W8"];IMyLaserAntenna ci=c9 as IMyLaserAntenna;if(ci!=null)return t0.po(ci); 
IMyRadioAntenna cj=c9 as IMyRadioAntenna;if(cj!=null)return sU.nB(cj.Radius)+"m";IMyBeacon ck=c9 as IMyBeacon;if(ck!=null)return 
sU.nB(ck.Radius)+"m";return M.T["W9"];}int lN=0;bool lO(sZ cl,bool cm,string cn,bool co){if(!co)lN=0;for(;lN<cl.pb();lN++){if(!rP. 
sa(20))return false;IMyTerminalBlock cp=cl.oR[lN];var cq=(cm?(cp.IsWorking?M.T["W9"]:M.T["W1"]):lM(cp));if(cn!=""&&cq.ToLower()!= 
cn)continue;if(cm)cq=lM(cp);var cr=cp.CustomName;cr=tc.si(cr,t8.qG*0.7f*mn.r4.rl);mn.Add(cr);mn.ra(cq);}return true;}}class sE:sN  
{public sE(){rL=5;rH="CmdDamage";}sZ lP;public override void Init(){lP=new sZ(rP,mn.qZ);}bool lQ=false;int lR=0;public override 
bool RunCmd(bool cs){var cu=mm.o6.StartsWith("damagex");var cv=mm.o6.EndsWith("noc");var cw=(!cv&&mm.o6.EndsWith("c"));float cx= 
100;if(!cs){lP.pa();lQ=false;lR=0;}if(!lP.p0(mm.o7,cs))return false;if(mm.oc.Count>0){if(!float.TryParse(mm.oc[0].ok,out cx))cx= 
100;}for(;lR<lP.pb();lR++){if(!rP.sa(30))return false;IMyTerminalBlock cy=lP.oR[lR];IMySlimBlock cz=cy.CubeGrid.GetCubeBlock(cy. 
Position);if(cz==null)continue;float cA=(cv?cz.MaxIntegrity:cz.BuildIntegrity);if(!cw)cA-=cz.CurrentDamage;float cB=100*(cA/cz. 
MaxIntegrity);if(cB>=cx)continue;lQ=true;mn.Add(tc.si(cz.FatBlock.DisplayNameText,t8.qG*0.69f*mn.r4.rl-70)+" ");if(!cu){mn.rb(sU. 
nB(cA)+" / ",0.69f);mn.Add(sU.nB(cz.MaxIntegrity));}mn.ra(' '+cB.ToString("0.0")+"%");mn.re(cB);}if(!lQ)mn.r7(M.T["D3"]);return 
true;}}class sF:sN {public sF(){rL=2;rH="CmdAmount";}sZ lS;public override void Init(){lS=new sZ(rP,mn.qZ);}bool lT;int lU=0;int 
lV=0;int lW=0;public override bool RunCmd(bool cC){if(!cC){lT=!mm.o6.EndsWith("x");if(mm.oc.Count==0)mm.oc.Add(new sX( 
"reactor,gatlingturret,missileturret,interiorturret,gatlinggun,launcherreload,launcher,oxygenerator"));lV=0;}for(;lV<mm.oc.Count; 
lV++){sX cD=mm.oc[lV];if(!cC){cD.om();lU=0;lW=0;}for(;lW<cD.ol.Count;lW++){if(lU==0){if(!cC){if(cD.ol[lW]=="")continue;lS.pa();} 
var cE=cD.ol[lW];if(!lS.p8(cE,mm.o7,cC))return false;lU++;cC=false;}if(!m6(cC))return false;cC=false;lU=0;}}return true;}int lX=0; 
int lY=0;double lZ=0;double l_=0;double m0=0;int m1=0;IMyTerminalBlock m2;IMyInventory m3;List<IMyInventoryItem>m4;string m5=""; 
bool m6(bool cF){if(!cF){lX=0;lY=0;}for(;lX<lS.pb();lX++){if(lY==0){if(!rP.sa(50))return false;m2=lS.oR[lX];m3=m2.GetInventory(0); 
if(m3==null)continue;lY++;cF=false;}if(!cF){m4=m3.GetItems();m5=(m4.Count>0?m4[0].Content.ToString():"");m1=0;lZ=0;l_=0;m0=0;}for( 
;m1<m4.Count;m1++){if(!rP.sa(30))return false;IMyInventoryItem cG=m4[m1];if(cG.Content.ToString()!=m5)m0+=(double)cG.Amount;else  
lZ+=(double)cG.Amount;}var cH=M.T["A1"];var cI=m2.CustomName;if(lZ>0&&(double)m3.CurrentVolume>0){double cJ=m0*(double)m3. 
CurrentVolume/(lZ+m0);l_=Math.Floor(lZ*((double)m3.MaxVolume-cJ)/(double)m3.CurrentVolume-cJ);cH=sU.nB(lZ)+" / "+(m0>0?"~":"")+sU. 
nB(l_);}cI=tc.si(cI,t8.qG*0.8f*mn.r4.rl);mn.Add(cI);mn.ra(cH);if(lT&&l_>0){double cK=100*lZ/l_;mn.re(cK);}lY=0;cF=false;}return 
true;}}class sG:sN {public sG(){rL=1;rH="CmdPosition";}public override bool RunCmd(bool cL){var cM=(mm.o6=="posxyz");var cN=(mm.o6 
=="posgps");IMyTerminalBlock cO=mo.mF.ql;if(mm.o7!=""&&mm.o7!="*"){cO=mn.r0.GetBlockWithName(mm.o7);if(cO==null){mn.r7("Pos: "+M.T 
["P1"]+": "+mm.o7);return true;}}if(cN){VRageMath.Vector3D cP=cO.GetPosition();mn.r7("GPS:"+M.T["P2"]+":"+cP.GetDim(0).ToString( 
"F2")+":"+cP.GetDim(1).ToString("F2")+":"+cP.GetDim(2).ToString("F2")+":");return true;}mn.Add(M.T["P2"]+": ");if(!cM){mn.ra(cO. 
GetPosition().ToString("F0"));return true;}mn.r7("");mn.Add(" X: ");mn.ra(cO.GetPosition().GetDim(0).ToString("F0"));mn.Add(" Y: " 
);mn.ra(cO.GetPosition().GetDim(1).ToString("F0"));mn.Add(" Z: ");mn.ra(cO.GetPosition().GetDim(2).ToString("F0"));return true;}} 
class sH:sN {public sH(){rL=5;rH="CmdDetails";}string m7="";sZ m8;public override void Init(){m8=new sZ(rP,mn.qZ);if(mm.oc.Count>0 
)m7=mm.oc[0].ok.Trim();}int m9=0;int ma=1;IMyTerminalBlock mb;public override bool RunCmd(bool cQ){if(mm.o7==""||mm.o7=="*"){mn.r7 
("Details: "+M.T["D1"]);return true;}if(!cQ){m8.pa();m9=0;ma=1;}if(m9==0){if(!m8.p0(mm.o7,cQ))return true;if(m8.pb()<=0){mn.r7( 
"Details: "+M.T["D2"]);return true;}m9++;cQ=false;}int cR=(mm.o6.EndsWith("x")?1:0);if(m9==1){if(!cQ){mb=m8.oR[0];mn.r7(mb. 
CustomName);}if(!mf(mb,cR,cQ))return false;m9++;cQ=false;}for(;ma<m8.pb();ma++){if(!cQ){mb=m8.oR[ma];mn.r7("");mn.r7(mb.CustomName 
);}if(!mf(mb,cR,cQ))return false;cQ=false;}return true;}string[]mc;int md=0;bool me=false;bool mf(IMyTerminalBlock cS,int cT,bool 
cU){if(!cU){mc=(cS.DetailedInfo+"\n"+cS.CustomInfo).Split('\n');md=cT;me=(m7=="");}for(;md<mc.Length;md++){if(!rP.sa(5))return 
false;if(mc[md]=="")continue;if(!me){if(!mc[md].Contains(m7))continue;me=true;}mn.r7("  "+mc[md]);}return true;}}class sI:sN { 
public sI(){rL=1;rH="CmdShipMass";}public override bool RunCmd(bool cV){var cW=mm.o6.EndsWith("base");double cX=0;if(mm.o7!="") 
double.TryParse(mm.o7.Trim(),out cX);int cY=mm.oc.Count;if(cY>0){var cZ=mm.oc[0].ok.Trim().ToLower();if(cZ!="")cX*=Math.Pow(1000.0 
,"kmgtpezy".IndexOf(cZ[0]));}double c_=(cW?mn.q_.oC:mn.q_.oB);if(!cW)mn.Add(M.T["SM1"]+" ");else mn.Add(M.T["SM2"]+" ");mn.ra(sU. 
nB(c_,true,'k')+"g ");if(cX>0)mn.re(c_/cX*100);return true;}}class sJ:sN {public sJ(){rL=1;rH="CmdDistance";}string mg="";string[] 
mh;Vector3D mi;string mj="";bool mk=false;public override void Init(){mk=false;if(mm.oc.Count<=0)return;mg=mm.oc[0].ok.Trim();mh= 
mg.Split(':');if(mh.Length<5||mh[0]!="GPS")return;double d0,d1,d2;if(!double.TryParse(mh[2],out d0))return;if(!double.TryParse(mh[ 
3],out d1))return;if(!double.TryParse(mh[4],out d2))return;mi=new Vector3D(d0,d1,d2);mj=mh[1];mk=true;}public override bool RunCmd 
(bool d3){if(!mk){mn.r7("Distance: "+M.T["DTU"]+" '"+mg+"'.");return true;}IMyTerminalBlock d4=mo.mF.ql;if(mm.o7!=""&&mm.o7!="*"){ 
d4=mn.r0.GetBlockWithName(mm.o7);if(d4==null){mn.r7("Distance: "+M.T["P1"]+": "+mm.o7);return true;}}double d5=Vector3D.Distance( 
d4.GetPosition(),mi);mn.Add(mj+": ");mn.ra(sU.nB(d5)+"m ");return true;}}class sK:sN {public sK(){rL=1;rH="CmdAltitude";}public 
override bool RunCmd(bool d6){var d7=(mm.o6.EndsWith("sea")?"sea":"ground");switch(d7){case "sea":mn.Add(M.T["ALT1"]);mn.ra(mn.q_. 
oE.ToString("F0")+" m");break;default:mn.Add(M.T["ALT2"]);mn.ra(mn.q_.oG.ToString("F0")+" m");break;}return true;}}class sL:sN { 
public sL(){rL=1;rH="CmdStopTask";}public override bool RunCmd(bool d8){double d9=0;if(mm.o6.Contains("best"))d9=mn.q_.os/mn.q_.ow 
;else d9=mn.q_.os/mn.q_.oz;double da=mn.q_.os/2*d9;if(mm.o6.Contains("time")){mn.Add(M.T["ST"]);if(double.IsNaN(d9)){mn.ra("N/A"); 
return true;}var db="";try{TimeSpan dc=TimeSpan.FromSeconds(d9);if((int)dc.TotalDays>0)db=" > 24h";else{if(dc.Hours>0)db=dc.Hours+ 
"h ";if(dc.Minutes>0||db!="")db+=dc.Minutes+"m ";db+=dc.Seconds+"s";}}catch{db="N/A";}mn.ra(db);return true;}mn.Add(M.T["SD"]);if( 
!double.IsNaN(da)&&!double.IsInfinity(da))mn.ra(sU.nB(da)+"m ");else mn.ra("N/A");return true;}}class sM:sN {public sM(){rL=1;rH= 
"CmdGravity";}public override bool RunCmd(bool dd){var de=(mm.o6.Contains("nat")?"n":(mm.o6.Contains("art")?"a":(mm.o6.Contains( 
"tot")?"t":"s")));Vector3D df;switch(de){case "n":mn.Add(M.T["G2"]+" ");df=mn.q_.oJ.GetNaturalGravity();mn.ra(df.Length().ToString 
("F1")+" m/s²");break;case "a":mn.Add(M.T["G3"]+" ");df=mn.q_.oJ.GetArtificialGravity();mn.ra(df.Length().ToString("F1")+" m/s²"); 
break;case "t":mn.Add(M.T["G1"]+" ");df=mn.q_.oJ.GetTotalGravity();mn.ra(df.Length().ToString("F1")+" m/s²");break;default:mn.Add( 
M.T["GN"]);mn.rb(" | ",0.33f);mn.rb(M.T["GA"]+" | ",0.66f);mn.ra(M.T["GT"],1.0f);mn.Add("");df=mn.q_.oJ.GetNaturalGravity();mn.rb( 
df.Length().ToString("F1")+" | ",0.33f);df=mn.q_.oJ.GetArtificialGravity();mn.rb(df.Length().ToString("F1")+" | ",0.66f);df=mn.q_. 
oJ.GetTotalGravity();mn.ra(df.Length().ToString("F1")+" ");break;}return true;}}class sN:ta {public t9 ml=null;protected sW mm; 
protected t8 mn;protected sQ mo;public sN(){rL=3600;rH="CommandTask";}public void mp(sQ dg,sW dh){mo=dg;mn=mo.mE;mm=dh;}public 
virtual bool RunCmd(bool di){mn.r7(M.T["UC"]+": '"+mm.o8+"'");return true;}public override bool Run(bool dj){ml=mn.r5(ml,mo.mF);if 
(!dj)mn.rg();return RunCmd(dj);}}class sO:ta {sS mq;t8 mr;string ms="";public sO(t8 dk,sS dl,string dm){rL=-1;rH="ArgScroll";ms=dm 
;mq=dl;mr=dk;}int mt;sZ mu;public override void Init(){mu=new sZ(rP,mr.qZ);}int mv=0;int mw=0;sW mx;public override bool Run(bool 
dn){if(!dn){mw=0;mu.pa();mx=new sW(rP);mv=0;}if(mw==0){if(!mx.oh(ms,dn))return false;if(mx.oc.Count>0){if(!int.TryParse(mx.oc[0]. 
ok,out mt))mt=1;else if(mt<1)mt=1;}if(mx.o6.EndsWith("up"))mt=-mt;else if(!mx.o6.EndsWith("down"))mt=0;mw++;dn=false;}if(mw==1){if 
(!mu.p8("textpanel",mx.o7,dn))return false;mw++;dn=false;}t6 dp;for(;mv<mu.pb();mv++){if(!rP.sa(20))return false;IMyTextPanel dq= 
mu.oR[mv]as IMyTextPanel;if(!mq.nf.TryGetValue(dq,out dp))continue;if(dp==null||dp.ql!=dq)continue;if(dp.qp)dp.qk.rF=10;if(mt>0)dp 
.qk.rE(mt);else if(mt<0)dp.qk.rD(-mt);else dp.qk.rG();dp.qu();}return true;}}class sP:ta {t8 my;sS mz;public int mA=0;public sP(t8 
dr,sS ds){rH="BootPanelsTask";rL=1;my=dr;mz=ds;if(!my.qK)mA=int.MaxValue;}public override bool Run(bool dt){if(mA>my.qL.Count){rT( 
);return true;}if(mA==0){mz.ng=false;}mC();mA++;return true;}public override void End(){mz.ng=true;}public void mB(){sT du=mz.nb; 
for(int dv=0;dv<du.nr();dv++){t6 dw=du.nt(dv);my.r5(dw.qk,dw);my.rg();my.rh(dw);}mA=(my.qK?0:int.MaxValue);}public void mC(){sT dx 
=mz.nb;for(int dy=0;dy<dx.nr();dy++){t6 dz=dx.nt(dy);my.r5(dz.qk,dz);my.rg();if(dz.ql.GetValueFloat("FontSize")>3f)continue;my.rc( 
M.T["B1"]);double dA=(double)mA/my.qL.Count*100;my.re(dA);if(mA==my.qL.Count){my.r7("");my.rc("Configurable Automatic LCDs");my.rc 
("by MMaster");}else my.r8(my.qL[mA]);my.rh(dz);}}public bool mD(){return mA<=my.qL.Count;}}class sQ:ta {public t8 mE;public t6 mF 
;public sR mG=null;string mH="N/A";public Dictionary<string,sN>mI=new Dictionary<string,sN>();public List<string>mJ=null;public sS 
mK;public bool mL{get{return mK.ng;}}public sQ(sS dB,t6 dC){rL=5;mF=dC;mK=dB;mE=dB.na;rH="PanelProcess";}sN mM(int dD){switch(dD){ 
case sV.nF:return new sw();case sV.nG:return new sk();case sV.nH:return new sl();case sV.nI:return new sm();case sV.nJ:return new 
sn();case sV.nK:return new so();case sV.nL:return new sp();case sV.nM:return new sr();case sV.nN:return new st();case sV.nO:return 
new su();case sV.nP:return new sv();case sV.nQ:return new sx();case sV.nR:return new sy();case sV.nS:return new sz();case sV.nT: 
return new sA();case sV.nU:return new sB();case sV.nV:return new sC();case sV.nW:return new sD();case sV.nX:return new sE();case 
sV.nY:return new sF();case sV.nZ:return new sG();case sV.n_:return new sH();case sV.o0:return new sI();case sV.o1:return new sJ(); 
case sV.o2:return new sK();case sV.o3:return new sL();case sV.o4:return new sM();default:return new sN();}}sW mN=null;sN mO(string 
dE,bool dF){if(!dF)mN=new sW(rP);if(!mN.oh(dE,dF))return null;sN dG=mM(mN.oa);dG.mp(this,mN);rP.s2(dG,0);return dG;}string mP=""; 
void mQ(){try{mP=mF.ql.CustomData;}catch{mP="";mF.ql.CustomData="";}mP=mP.Replace("\\\n","");}int mR=0;int mS=0;List<string>mT= 
null;HashSet<string>mU=new HashSet<string>();int mV=0;bool mW(bool dH){if(!dH){char[]dI={';','\n' };var dJ=mP.Replace("\\;","\f"); 
mT=new List<string>(dJ.Split(dI,StringSplitOptions.RemoveEmptyEntries));mU.Clear();mR=0;mS=0;mV=0;}while(mR<mT.Count){if(!rP.sa( 
500))return false;if(mT[mR].StartsWith("//")){mT.RemoveAt(mR);continue;}mT[mR]=mT[mR].Replace('\f',';');if(!mI.ContainsKey(mT[mR]) 
){if(mV!=1)dH=false;mV=1;sN dK=mO(mT[mR],dH);if(dK==null)return false;dH=false;mI.Add(mT[mR],dK);mV=0;}if(!mU.Contains(mT[mR]))mU. 
Add(mT[mR]);mR++;}if(mJ!=null){sN dL;while(mS<mJ.Count){if(!rP.sa(7))return false;if(!mU.Contains(mJ[mS]))if(mI.TryGetValue(mJ[mS] 
,out dL)){dL.rT();mI.Remove(mJ[mS]);}mS++;}}mJ=mT;return true;}public override void End(){if(mJ!=null){sN dM;for(int dN=0;dN<mJ. 
Count;dN++){if(mI.TryGetValue(mJ[dN],out dM))dM.rT();}mJ=null;}if(mG!=null){mG.rT();mG=null;}}string mX="";bool mY=false;public 
override bool Run(bool dO){if(mF.qj.qz()<=0){rT();return true;}if(!dO){mF.qk=mE.r5(mF.qk,mF);mQ();if(mF.ql.CustomName!=mX)mY=true; 
else mY=false;mX=mF.ql.CustomName;}if(mP!=mH){if(!mW(dO))return false;if(mP==""){mE.rg();mE.r7(M.T["H1"]);mE.rh(mF);return true;} 
mY=true;}mH=mP;if(mG!=null&&mY){rP.s3(mG);mG.n2();rP.s2(mG,0);}else if(mG==null){mG=new sR(this);rP.s2(mG,0);}return true;}}class 
sR:ta {public t8 mZ;public t6 m_;sQ n0;public sR(sQ dP){n0=dP;mZ=n0.mE;m_=n0.mF;rL=0.5;rH="PanelDisplay";}double n1=0;public void 
n2(){n1=0;}int n3=0;int n4=0;bool n5=true;double n6=double.MaxValue;int n7=0;public override bool Run(bool dQ){rP.s4("Start");sN 
dR;if(!dQ&&(!n0.mL||n0.mJ==null||n0.mJ.Count<=0))return true;rP.s4("A");if(n0.mK.n9>5)return rR(0);rP.s4("B");if(!dQ){n4=0;n5= 
false;n6=double.MaxValue;n7=0;}rP.s4("C");if(n7==0){while(n4<n0.mJ.Count){if(!rP.sa(5))return false;rP.s4("D");if(n0.mI. 
TryGetValue(n0.mJ[n4],out dR)){if(!dR.rN)return rR(dR.rI-rP.rX+0.001);if(dR.rJ>n1)n5=true;if(dR.rI<n6)n6=dR.rI;}n4++;}n7++;dQ= 
false;}rP.s4("E");double dS=n6-rP.rX+0.001;if(!n5&&!m_.qq())return rR(dS);rP.s4("F");mZ.r6(m_);rP.s4("G");if(n5){if(!dQ){n1=rP.rX; 
mZ.rg();var dT=m_.ql.CustomName;dT=(dT.Contains("#")?dT.Substring(dT.LastIndexOf('#')+1):"");if(dT!="")mZ.r7(dT);n3=0;}while(n3<n0 
.mJ.Count){if(!rP.sa(7))return false;if(!n0.mI.TryGetValue(n0.mJ[n3],out dR)){mZ.r7("ERR: No cmd task ("+n0.mJ[n3]+")");n3++; 
continue;}mZ.r9(dR.ml.rx());n3++;}}mZ.rh(m_);n0.mK.n9++;if(rL<dS&&!m_.qq())return rR(dS);return true;}}class sS:ta {public int n9= 
0;public t8 na;public sT nb=new sT();sZ nc;sZ nd;Dictionary<t6,sQ>ne=new Dictionary<t6,sQ>();public Dictionary<IMyTextPanel,t6>nf= 
new Dictionary<IMyTextPanel,t6>();public bool ng=false;sP nh=null;public sS(t8 dU){rL=5;na=dU;rH="ProcessPanels";}public override 
void Init(){nc=new sZ(rP,na.qZ);nd=new sZ(rP,na.qZ);nh=new sP(na,this);}int ni=0;bool nj(bool dV){if(!dV)ni=0;if(ni==0){if(!nc.p8( 
"textpanel",na.qI,dV))return false;ni++;dV=false;}if(ni==1){if(na.qI=="T:[LCD]"&&"T:!LCD!"!="")if(!nc.p8("textpanel","T:!LCD!",dV) 
)return false;ni++;dV=false;}return true;}string nk(IMyTextPanel dW){return dW.CustomName+" "+dW.NumberInGrid+" "+dW.GetPosition() 
.ToString("F0");}int nl=0;int nm=0;public override bool Run(bool dX){if(!dX){nc.pa();nl=0;nm=0;}if(!nj(dX))return false;while(nl< 
nc.pb()){if(!rP.sa(20))return false;IMyTextPanel dY=(nc.oR[nl]as IMyTextPanel);if(dY==null||!dY.IsWorking){nc.oR.RemoveAt(nl); 
continue;}if(nf.ContainsKey(dY)){nl++;continue;}t6 dZ=null;var d_=nk(dY);int e0=d_.IndexOf("!LINK:");if(e0<0||d_.Length==e0+6){dZ= 
new t6(na,d_);dZ.qj.qx(d_,dY);sQ e1=new sQ(this,dZ);rP.s2(e1,0);ne.Add(dZ,e1);nb.nq(d_,dZ);nf.Add(dY,dZ);nl++;continue;}d_=d_. 
Substring(e0+6);string[]e2=d_.Split(' ');var e3=e2[0];dZ=nb.ns(e3);if(dZ==null){dZ=new t6(na,e3);nb.nq(e3,dZ);sQ e1=new sQ(this,dZ 
);rP.s2(e1,0);ne.Add(dZ,e1);}dZ.qj.qx(d_,dY);nf.Add(dY,dZ);nl++;}while(nm<nd.pb()){if(!rP.sa(300))return false;IMyTextPanel dY=nd. 
oR[nm]as IMyTextPanel;if(dY==null)continue;if(!nc.oR.Contains(dY)){t6 dZ=nf[dY];var d_=dZ.qo;dZ.qj.qy(d_);nf.Remove(dY);if(dZ.qj. 
qz()<=0){sQ e1;if(ne.TryGetValue(dZ,out e1)){nb.nu(d_);ne.Remove(dZ);e1.rT();}}}nm++;}nd.pa();nd.p9(nc);if(!nh.rM&&nh.mD())rP.s2( 
nh,0);return true;}public bool nn(string e9){rP.s4("RunCommand: "+e9);var ea=e9.ToLower();if(ea=="clear"){rP.s4("ClearLCDs");nh.mB 
();if(!nh.rM)rP.s2(nh,0);return true;}if(ea=="boot"){rP.s4("Boot, isRunning"+nh.rM);nh.mA=0;if(!nh.rM)rP.s2(nh,0);return true;}if( 
ea.StartsWith("scroll")){sO eb=new sO(na,this,e9);rP.s2(eb,0);return true;}return false;}}public class sT {Dictionary<string,t6>no 
=new Dictionary<string,t6>();List<string>np=new List<string>();public void nq(string ec,t6 ed){if(!no.ContainsKey(ec)){np.Add(ec); 
no.Add(ec,ed);}}public int nr(){return no.Count;}public t6 ns(string ee){if(no.ContainsKey(ee))return no[ee];return null;}public 
t6 nt(int ef){return no[np[ef]];}public void nu(string eg){no.Remove(eg);np.Remove(eg);}public void nv(){np.Clear();no.Clear();} 
public void nw(){np.Sort();}}public static class sU {public static string nx(IMyInventoryItem eh){var ei=eh.Content.TypeId. 
ToString();ei=ei.Substring(ei.LastIndexOf('_')+1);return eh.Content.SubtypeName+" "+ei;}public static void ny(string ej,out string 
ek,out string el){int em=ej.LastIndexOf(' ');if(em>=0){ek=ej.Substring(0,em);el=ej.Substring(em+1);return;}ek=ej;el="";}public 
static string nz(string en){string eo,ep;ny(en,out eo,out ep);return nz(eo,ep);}public static string nz(string eq,string er){t1 es 
=MMItems.pw(eq,er);if(es!=null){if(es.pA!="")return es.pA;return es.px;}return System.Text.RegularExpressions.Regex.Replace(eq, 
"([a-z])([A-Z])","$1 $2");}public static void nA(ref string et,ref string eu){var ev=et.ToLower();t1 ew;if(MMItems.pv.TryGetValue( 
ev,out ew)){et=ew.px;eu=ew.py;return;}ew=MMItems.pw(et,eu);if(ew!=null){et=ew.px;if(eu=="Ore"||eu=="Ingot")return;eu=ew.py;}} 
public static string nB(double ex,bool ey=true,char ez=' '){if(!ey)return ex.ToString("#,###,###,###,###,###,###,###,###,###");var 
eA=" kMGTPEZY";double eB=ex;int eC=eA.IndexOf(ez);int eD=(eC<0?0:eC);while(eB>=1000&&eD+1<eA.Length){eB/=1000;eD++;}var eE=Math. 
Round(eB,1,MidpointRounding.AwayFromZero).ToString();if(eD>0)eE+=" "+eA[eD];return eE;}public static string nC(double eF){return( 
Math.Floor(eF*10)/10).ToString("F1");}}public static class sV {public const int nF=1;public const int nG=2;public const int nH=3; 
public const int nI=4;public const int nJ=5;public const int nK=6;public const int nL=7;public const int nM=8;public const int nN= 
9;public const int nO=10;public const int nP=11;public const int nQ=12;public const int nR=13;public const int nS=14;public const 
int nT=15;public const int nU=16;public const int nV=17;public const int nW=18;public const int nX=19;public const int nY=20; 
public const int nZ=21;public const int n_=22;public const int o0=23;public const int o1=24;public const int o2=25;public const 
int o3=26;public const int o4=27;}public class sW {tb o5;public string o6="";public string o7="";public string o8="";public string 
o9="";public int oa=0;int ob(){if(o6.StartsWith("inventory")||o6=="missing"||o6.StartsWith("invlist"))return sV.nG;if(o6. 
StartsWith("cargo"))return sV.nH;if(o6.StartsWith("mass"))return sV.nI;if(o6.StartsWith("shipmass"))return sV.o0;if(o6=="oxygen") 
return sV.nJ;if(o6=="tanks")return sV.nK;if(o6.StartsWith("powertime"))return sV.nL;if(o6.StartsWith("powerused"))return sV.nM;if( 
o6.StartsWith("power"))return sV.nN;if(o6.StartsWith("speed"))return sV.nO;if(o6.StartsWith("accel"))return sV.nP;if(o6.StartsWith 
("alti"))return sV.o2;if(o6.StartsWith("charge"))return sV.nQ;if(o6.StartsWith("time")||o6.StartsWith("date"))return sV.nR;if(o6. 
StartsWith("countdown"))return sV.nS;if(o6=="echo"||o6=="center"||o6=="right")return sV.nF;if(o6.StartsWith("textlcd"))return sV. 
nT;if(o6.EndsWith("count"))return sV.nU;if(o6.StartsWith("dampeners")||o6.StartsWith("occupied"))return sV.nV;if(o6.StartsWith( 
"working"))return sV.nW;if(o6.StartsWith("damage"))return sV.nX;if(o6.StartsWith("amount"))return sV.nY;if(o6.StartsWith("pos")) 
return sV.nZ;if(o6.StartsWith("distance"))return sV.o1;if(o6.StartsWith("details"))return sV.n_;if(o6.StartsWith("stop"))return sV 
.o3;if(o6.StartsWith("gravity"))return sV.o4;return 0;}public sW(tb eG){o5=eG;}public List<sX>oc=new List<sX>();string[]od=null; 
string oe="";bool of=false;int og=1;public bool oh(string eH,bool eI){if(!eI){oa=0;o7="";o6="";o8=eH.TrimStart(' ');oc.Clear();if( 
o8=="")return true;int eJ=o8.IndexOf(' ');if(eJ<0||eJ>=o8.Length-1)o9="";else o9=o8.Substring(eJ+1);od=o8.Split(' ');oe="";of= 
false;o6=od[0].ToLower();og=1;}for(;og<od.Length;og++){if(!o5.sa(40))return false;var eK=od[og];if(eK=="")continue;if(eK[0]=='{'&& 
eK[eK.Length-1]=='}'){eK=eK.Substring(1,eK.Length-2);if(eK=="")continue;if(o7=="")o7=eK;else oc.Add(new sX(eK));continue;}if(eK[0] 
=='{'){of=true;oe=eK.Substring(1);continue;}if(eK[eK.Length-1]=='}'){of=false;oe+=' '+eK.Substring(0,eK.Length-1);if(o7=="")o7=oe; 
else oc.Add(new sX(oe));continue;}if(of){if(oe.Length!=0)oe+=' ';oe+=eK;continue;}if(o7=="")o7=eK;else oc.Add(new sX(eK));}oa=ob() 
;return true;}}public class sX {public string oi="";public string oj="";public string ok="";public List<string>ol=new List<string> 
();public sX(string eL){ok=eL;}public void om(){if(ok==""||oi!=""||oj!=""||ol.Count>0)return;var eM=ok.Trim();if(eM[0]=='+'||eM[0] 
=='-'){oi+=eM[0];eM=ok.Substring(1);}string[]eN=eM.Split('/');var eO=eN[0];if(eN.Length>1){oj=eN[0];eO=eN[1];}else oj="";if(eO. 
Length>0){string[]eP=eO.Split(',');for(int eQ=0;eQ<eP.Length;eQ++)if(eP[eQ]!="")ol.Add(eP[eQ]);}}}public class sY:ta { 
MyShipVelocities on;public VRageMath.Vector3D oo{get{return on.LinearVelocity;}}public VRageMath.Vector3D oq{get{return on. 
AngularVelocity;}}double or=0;public double os{get{if(oH!=null)return oH.GetShipSpeed();else return or;}}double ot=0;public double 
ou{get{return ot;}}double ov=0;public double ow{get{return ov;}}double ox=0;double oy=0;public double oz{get{return ox;}} 
MyShipMass oA;public double oB{get{return oA.TotalMass;}}public double oC{get{return oA.BaseMass;}}double oD=double.NaN;public 
double oE{get{return oD;}}double oF=double.NaN;public double oG{get{return oF;}}IMyShipController oH=null;IMySlimBlock oI=null; 
public IMyShipController oJ{get{return oH;}}VRageMath.Vector3D oK;public sY(tb eR){rH="ShipMgr";rP=eR;oK=rP.s0.Me.GetPosition();rL 
=0.5;}List<IMyTerminalBlock>oL=new List<IMyTerminalBlock>();int oM=0;public override bool Run(bool eS){if(!eS){oL.Clear();rP.s0. 
GridTerminalSystem.GetBlocksOfType<IMyShipController>(oL);oM=0;if(oH!=null&&oH.CubeGrid.GetCubeBlock(oH.Position)!=oI)oH=null;}if( 
oL.Count>0){for(;oM<oL.Count;oM++){if(!rP.sa(20))return false;IMyShipController eT=oL[oM]as IMyShipController;if(eT.GetValueBool( 
"MainCockpit")||eT.IsUnderControl){oH=eT;oI=eT.CubeGrid.GetCubeBlock(eT.Position);if(eT.GetValueBool("MainCockpit")){oM=oL.Count; 
break;}}}if(oH==null){oH=oL[0]as IMyShipController;oI=oH.CubeGrid.GetCubeBlock(oH.Position);}oA=oH.CalculateShipMass();if(!oH. 
TryGetPlanetElevation(MyPlanetElevation.Sealevel,out oD))oD=double.NaN;if(!oH.TryGetPlanetElevation(MyPlanetElevation.Surface,out 
oF))oF=double.NaN;on=oH.GetShipVelocities();}double eU=or;or=oo.Length();ot=(or-eU)/rK;if(-ot>ov)ov=-ot;if(-ot>ox){ox=-ot;oy=rP.rX 
;}if(rP.rX-oy>5&&-ot>0.1)ox-=(ox+ot)*0.3f;return true;}}public class sZ {tb oN=null;s_ oO;IMyCubeGrid oP{get{return oN.s0.Me. 
CubeGrid;}}IMyGridTerminalSystem oQ{get{return oN.s0.GridTerminalSystem;}}public List<IMyTerminalBlock>oR=new List< 
IMyTerminalBlock>();public sZ(tb eV,s_ eW){oN=eV;oO=eW;}int oS=0;public double oT(ref double eX,ref double eY,bool eZ){if(!eZ)oS=0 
;for(;oS<oR.Count;oS++){if(!oN.sa(4))return Double.NaN;IMyInventory e_=oR[oS].GetInventory(0);if(e_==null)continue;eX+=(double)e_. 
CurrentVolume;eY+=(double)e_.MaxVolume;}eX*=1000;eY*=1000;return(eY>0?eX/eY*100:100);}int oU=0;double oV=0;public double oW(bool 
f0){if(!f0){oU=0;oV=0;}for(;oU<oR.Count;oU++){if(!oN.sa(6))return Double.NaN;for(int f1=0;f1<2;f1++){IMyInventory f2=oR[oU]. 
GetInventory(f1);if(f2==null)continue;oV+=(double)f2.CurrentMass;}}return oV*1000;}int oX=0;bool oY(bool f3=false){if(!f3)oX=0; 
while(oX<oR.Count){if(!oN.sa(4))return false;if(oR[oX].CubeGrid!=oP){oR.RemoveAt(oX);continue;}oX++;}return true;}List< 
IMyBlockGroup>oZ=new List<IMyBlockGroup>();int o_=0;public bool p0(string f4,bool f5){int f6=f4.IndexOf(':');var f7=(f6>=1&&f6<=2? 
f4.Substring(0,f6):"");var f8=f7.Contains("T");if(f7!="")f4=f4.Substring(f6+1);if(f4==""||f4=="*"){if(!f5){var f9=new List< 
IMyTerminalBlock>();oQ.GetBlocks(f9);oR.AddList(f9);}if(f8)if(!oY(f5))return false;return true;}var fa=(f7.Contains("G")?f4.Trim() 
.ToLower():"");if(fa!=""){if(!f5){oZ.Clear();oQ.GetBlockGroups(oZ);o_=0;}for(;o_<oZ.Count;o_++){IMyBlockGroup fb=oZ[o_];if(fb.Name 
.ToLower()==fa){if(!f5)fb.GetBlocks(oR);if(f8)if(!oY(f5))return false;return true;}}return true;}if(!f5)oQ.SearchBlocksOfName(f4, 
oR);if(f8)if(!oY(f5))return false;return true;}List<IMyBlockGroup>p1=new List<IMyBlockGroup>();List<IMyTerminalBlock>p2=new List< 
IMyTerminalBlock>();int p3=0;int p4=0;public bool p5(string fc,string fd,bool fe,bool ff){if(!ff){p1.Clear();oQ.GetBlockGroups(p1) 
;p3=0;}for(;p3<p1.Count;p3++){IMyBlockGroup fg=p1[p3];if(fg.Name.ToLower()==fd){if(!ff){p4=0;p2.Clear();fg.GetBlocks(p2);}else ff= 
false;for(;p4<p2.Count;p4++){if(!oN.sa(6))return false;if(fe&&p2[p4].CubeGrid!=oP)continue;if(oO.pk(p2[p4],fc))oR.Add(p2[p4]);} 
return true;}}return true;}List<IMyTerminalBlock>p6=new List<IMyTerminalBlock>();int p7=0;public bool p8(string fh,string fi,bool 
fj){int fk=fi.IndexOf(':');var fl=(fk>=1&&fk<=2?fi.Substring(0,fk):"");var fm=fl.Contains("T");if(fl!="")fi=fi.Substring(fk+1);if( 
!fj){p6.Clear();p7=0;}var fn=(fl.Contains("G")?fi.Trim().ToLower():"");if(fn!=""){if(!p5(fh,fn,fm,fj))return false;return true;}if 
(!fj)oO.pj(ref p6,fh);if(fi==""||fi=="*"){if(!fj)oR.AddList(p6);if(fm)if(!oY(fj))return false;return true;}for(;p7<p6.Count;p7++){ 
if(!oN.sa(4))return false;if(fm&&p6[p7].CubeGrid!=oP)continue;if(p6[p7].CustomName.Contains(fi))oR.Add(p6[p7]);}return true;} 
public void p9(sZ fo){oR.AddList(fo.oR);}public void pa(){oR.Clear();}public int pb(){return oR.Count;}}public class s_ {tb pc; 
public MyGridProgram pd{get{return pc.s0;}}public IMyGridTerminalSystem pe{get{return pc.s0.GridTerminalSystem;}} 
IMyGridTerminalSystem pf=null;Dictionary<string,Action<List<IMyTerminalBlock>,Func<IMyTerminalBlock,bool>>>pg=null;public s_(tb fp 
){pc=fp;pf=pc.s0.GridTerminalSystem;}public void ph(){if(pg!=null&&pe.GetBlocksOfType<IMyCargoContainer> ==pg["CargoContainer"]) 
return;pg=new Dictionary<string,Action<List<IMyTerminalBlock>,Func<IMyTerminalBlock,bool>>>(){{"CargoContainer",pe.GetBlocksOfType 
<IMyCargoContainer>},{"TextPanel",pe.GetBlocksOfType<IMyTextPanel>},{"Assembler",pe.GetBlocksOfType<IMyAssembler>},{"Refinery",pe. 
GetBlocksOfType<IMyRefinery>},{"Reactor",pe.GetBlocksOfType<IMyReactor>},{"SolarPanel",pe.GetBlocksOfType<IMySolarPanel>},{ 
"BatteryBlock",pe.GetBlocksOfType<IMyBatteryBlock>},{"Beacon",pe.GetBlocksOfType<IMyBeacon>},{"RadioAntenna",pe.GetBlocksOfType< 
IMyRadioAntenna>},{"AirVent",pe.GetBlocksOfType<IMyAirVent>},{"ConveyorSorter",pe.GetBlocksOfType<IMyConveyorSorter>},{ 
"OxygenTank",pe.GetBlocksOfType<IMyOxygenTank>},{"OxygenGenerator",pe.GetBlocksOfType<IMyOxygenGenerator>},{"OxygenFarm",pe. 
GetBlocksOfType<IMyOxygenFarm>},{"LaserAntenna",pe.GetBlocksOfType<IMyLaserAntenna>},{"Thrust",pe.GetBlocksOfType<IMyThrust>},{ 
"Gyro",pe.GetBlocksOfType<IMyGyro>},{"SensorBlock",pe.GetBlocksOfType<IMySensorBlock>},{"ShipConnector",pe.GetBlocksOfType< 
IMyShipConnector>},{"ReflectorLight",pe.GetBlocksOfType<IMyReflectorLight>},{"InteriorLight",pe.GetBlocksOfType<IMyInteriorLight>} 
,{"LandingGear",pe.GetBlocksOfType<IMyLandingGear>},{"ProgrammableBlock",pe.GetBlocksOfType<IMyProgrammableBlock>},{"TimerBlock", 
pe.GetBlocksOfType<IMyTimerBlock>},{"MotorStator",pe.GetBlocksOfType<IMyMotorStator>},{"PistonBase",pe.GetBlocksOfType< 
IMyPistonBase>},{"Projector",pe.GetBlocksOfType<IMyProjector>},{"ShipMergeBlock",pe.GetBlocksOfType<IMyShipMergeBlock>},{ 
"SoundBlock",pe.GetBlocksOfType<IMySoundBlock>},{"Collector",pe.GetBlocksOfType<IMyCollector>},{"JumpDrive",pe.GetBlocksOfType< 
IMyJumpDrive>},{"Door",pe.GetBlocksOfType<IMyDoor>},{"GravityGeneratorSphere",pe.GetBlocksOfType<IMyGravityGeneratorSphere>},{ 
"GravityGenerator",pe.GetBlocksOfType<IMyGravityGenerator>},{"ShipDrill",pe.GetBlocksOfType<IMyShipDrill>},{"ShipGrinder",pe. 
GetBlocksOfType<IMyShipGrinder>},{"ShipWelder",pe.GetBlocksOfType<IMyShipWelder>},{"LargeGatlingTurret",pe.GetBlocksOfType< 
IMyLargeGatlingTurret>},{"LargeInteriorTurret",pe.GetBlocksOfType<IMyLargeInteriorTurret>},{"LargeMissileTurret",pe. 
GetBlocksOfType<IMyLargeMissileTurret>},{"SmallGatlingGun",pe.GetBlocksOfType<IMySmallGatlingGun>},{"SmallMissileLauncherReload", 
pe.GetBlocksOfType<IMySmallMissileLauncherReload>},{"SmallMissileLauncher",pe.GetBlocksOfType<IMySmallMissileLauncher>},{ 
"VirtualMass",pe.GetBlocksOfType<IMyVirtualMass>},{"Warhead",pe.GetBlocksOfType<IMyWarhead>},{"FunctionalBlock",pe.GetBlocksOfType 
<IMyFunctionalBlock>},{"LightingBlock",pe.GetBlocksOfType<IMyLightingBlock>},{"ControlPanel",pe.GetBlocksOfType<IMyControlPanel>}, 
{"Cockpit",pe.GetBlocksOfType<IMyCockpit>},{"MedicalRoom",pe.GetBlocksOfType<IMyMedicalRoom>},{"RemoteControl",pe.GetBlocksOfType< 
IMyRemoteControl>},{"ButtonPanel",pe.GetBlocksOfType<IMyButtonPanel>},{"CameraBlock",pe.GetBlocksOfType<IMyCameraBlock>},{ 
"OreDetector",pe.GetBlocksOfType<IMyOreDetector>},{"ShipController",pe.GetBlocksOfType<IMyShipController>}};}public void pi(ref 
List<IMyTerminalBlock>fq,string fr){Action<List<IMyTerminalBlock>,Func<IMyTerminalBlock,bool>>fs=null;if(pg.TryGetValue(fr,out fs) 
)fs(fq,null);else{if(fr=="CryoChamber"){pe.GetBlocksOfType<IMyCockpit>(fq,ft=>ft.BlockDefinition.ToString().Contains("Cryo")); 
return;}}}public void pj(ref List<IMyTerminalBlock>fu,string fv){pi(ref fu,pl(fv.Trim()));}public bool pk(IMyTerminalBlock fw, 
string fx){var fy=pl(fx);switch(fy){case "FunctionalBlock":return true;case "ShipController":return(fw as IMyShipController!=null) 
;default:return fw.BlockDefinition.ToString().Contains(pl(fx));}}public string pl(string fz){fz=fz.ToLower();if(fz.StartsWith( 
"carg")||fz.StartsWith("conta"))return "CargoContainer";if(fz.StartsWith("text")||fz.StartsWith("lcd"))return "TextPanel";if(fz. 
StartsWith("ass"))return "Assembler";if(fz.StartsWith("refi"))return "Refinery";if(fz.StartsWith("reac"))return "Reactor";if(fz. 
StartsWith("solar"))return "SolarPanel";if(fz.StartsWith("bat"))return "BatteryBlock";if(fz.StartsWith("bea"))return "Beacon";if( 
fz.Contains("vent"))return "AirVent";if(fz.Contains("sorter"))return "ConveyorSorter";if(fz.Contains("tank"))return "OxygenTank"; 
if(fz.Contains("farm")&&fz.Contains("oxy"))return "OxygenFarm";if(fz.Contains("gene")&&fz.Contains("oxy"))return "OxygenGenerator" 
;if(fz.Contains("cryo"))return "CryoChamber";if(fz=="laserantenna")return "LaserAntenna";if(fz.Contains("antenna"))return 
"RadioAntenna";if(fz.StartsWith("thrust"))return "Thrust";if(fz.StartsWith("gyro"))return "Gyro";if(fz.StartsWith("sensor"))return 
"SensorBlock";if(fz.Contains("connector"))return "ShipConnector";if(fz.StartsWith("reflector"))return "ReflectorLight";if((fz. 
StartsWith("inter")&&fz.EndsWith("light")))return "InteriorLight";if(fz.StartsWith("land"))return "LandingGear";if(fz.StartsWith( 
"program"))return "ProgrammableBlock";if(fz.StartsWith("timer"))return "TimerBlock";if(fz.StartsWith("motor"))return "MotorStator" 
;if(fz.StartsWith("piston"))return "PistonBase";if(fz.StartsWith("proj"))return "Projector";if(fz.Contains("merge"))return 
"ShipMergeBlock";if(fz.StartsWith("sound"))return "SoundBlock";if(fz.StartsWith("col"))return "Collector";if(fz.Contains("jump")) 
return "JumpDrive";if(fz=="door")return "Door";if((fz.Contains("grav")&&fz.Contains("sphe")))return "GravityGeneratorSphere";if(fz 
.Contains("grav"))return "GravityGenerator";if(fz.EndsWith("drill"))return "ShipDrill";if(fz.Contains("grind"))return 
"ShipGrinder";if(fz.EndsWith("welder"))return "ShipWelder";if((fz.Contains("turret")&&fz.Contains("gatl")))return 
"LargeGatlingTurret";if((fz.Contains("turret")&&fz.Contains("inter")))return "LargeInteriorTurret";if((fz.Contains("turret")&&fz. 
Contains("miss")))return "LargeMissileTurret";if(fz.Contains("gatl"))return "SmallGatlingGun";if((fz.Contains("launcher")&&fz. 
Contains("reload")))return "SmallMissileLauncherReload";if((fz.Contains("launcher")))return "SmallMissileLauncher";if(fz.Contains( 
"mass"))return "VirtualMass";if(fz=="warhead")return "Warhead";if(fz.StartsWith("func"))return "FunctionalBlock";if(fz=="shipctrl" 
)return "ShipController";if(fz.StartsWith("light"))return "LightingBlock";if(fz.StartsWith("contr"))return "ControlPanel";if(fz. 
StartsWith("coc"))return "Cockpit";if(fz.StartsWith("medi"))return "MedicalRoom";if(fz.StartsWith("remote"))return "RemoteControl" 
;if(fz.StartsWith("but"))return "ButtonPanel";if(fz.StartsWith("cam"))return "CameraBlock";if(fz.Contains("detect"))return 
"OreDetector";return "Unknown";}}public static class t0 {public static List<double>pm(IMyTerminalBlock fA,int fB=-1){var fC=new 
List<double>();string[]fD=fA.DetailedInfo.Split('\n');int fE=Math.Min(fD.Length,(fB>0?fB:fD.Length));for(int fF=0;fF<fE;fF++){ 
string[]fG=fD[fF].Split(':');if(fG.Length<2){fG=fD[fF].Split('r');if(fG.Length<2)fG=fD[fF].Split('x');}var fH=(fG.Length<2?fG[0]: 
fG[1]);string[]fI=fH.Trim().Split(' ');var fJ=fI[0].Trim();var fK=(fI.Length>1&&fI[1].Length>1?fI[1][0]:'.');double fL;if(Double. 
TryParse(fJ,out fL)){double fM=fL*Math.Pow(1000.0,".kMGTPEZY".IndexOf(fK));fC.Add(fM);}}return fC;}public static string pn( 
IMyBatteryBlock fN){var fO="";if(fN.GetValueBool("Recharge"))fO="(+) ";else if(fN.GetValueBool("Discharge"))fO="(-) ";else fO= 
"(±) ";return fO+sU.nC((fN.CurrentStoredPower/fN.MaxStoredPower)*100.0f)+"%";}public static string po(IMyLaserAntenna fP){string[] 
fQ=fP.DetailedInfo.Split('\n');return fQ[fQ.Length-1].Split(' ')[0].ToUpper();}public static double pp(IMyJumpDrive fR,out double 
fS,out double fT){List<double>fU=pm(fR,5);if(fU.Count<4){fT=0;fS=0;return 0;}fT=fU[1];fS=fU[3];return(fT>0?fS/fT*100:0);}public 
static double pq(IMyJumpDrive fV){List<double>fW=pm(fV,5);double fX=0,fY=0;if(fW.Count<4)return 0;fX=fW[1];fY=fW[3];return(fX>0?fY 
/fX*100:0);}public static string pr(IMyLandingGear fZ){var f_="udoesnp";string[]g0=fZ.DetailedInfo.Split('\n');var g1=g0[g0.Length 
-1];string[]g2=g1.Split(':');string g3;if(g2.Length<2){g3=g2[0].Trim().ToLower();if(g3.StartsWith("slot status"))g3=g3.Substring( 
11);}else g3=g2[1].Trim().ToLower();if(g3=="")return M.T["W8"];if(g3.Split(' ').Length>1)return M.T["W10"];if((f_.IndexOf(g3[0])<0 
)&&!g3.StartsWith("au"))return M.T["W7"];return M.T["W8"];}}public static class MMItems {public static Dictionary<string,t1>ps=new 
Dictionary<string,t1>();static Dictionary<string,t1>pt=new Dictionary<string,t1>();public static List<string>pu=new List<string>() 
;public static Dictionary<string,t1>pv=new Dictionary<string,t1>();public static void Add(string g4,string g5,int g6,string g7, 
string g8,bool g9){if(g5=="Ammo")g5="AmmoMagazine";else if(g5=="Tool")g5="PhysicalGunObject";var ga=g4+' '+g5;t1 gb=new t1(g4,g5, 
g6,g7,g8,g9);ps.Add(ga,gb);if(!pt.ContainsKey(g4))pt.Add(g4,gb);if(g8!="")pv.Add(g8.ToLower(),gb);pu.Add(ga);}public static t1 pw( 
string gc="",string gd=""){if(ps.ContainsKey(gc+" "+gd))return ps[gc+" "+gd];if(gd==""){t1 ge=null;pt.TryGetValue(gc,out ge); 
return ge;}if(gc=="")for(int gf=0;gf<ps.Count;gf++){t1 ge=ps[pu[gf]];if(gd==ge.py)return ge;}return null;}}public class t1 {public 
string px;public string py;public int pz;public string pA;public string pB;public bool pC;public t1(string gh,string gi,int gj=0, 
string gk="",string gl="",bool gm=true){px=gh;py=gi;pz=gj;pA=gk;pB=gl;pC=gm;}}public class t2 {static Dictionary<string,string>pD= 
new Dictionary<string,string>(){{"ingot","ingot" },{"ore","ore" },{"component","component" },{"tool","physicalgunobject" },{"ammo" 
,"ammomagazine" },{"oxygen","oxygencontainerobject" },{"gas","gascontainerobject" }};tb pE;t4 pF;t4 pG;t4 pH;bool pI;public t4 pJ; 
public t2(tb gn,int go=20){pF=new t4();pG=new t4();pH=new t4();pI=false;pJ=new t4();pE=gn;}public void pK(){pH.q6();pG.q6();pF.q6( 
);pI=false;pJ.q6();}public void pL(string gp,bool gq=false,int gr=1,int gs=-1){if(gp==""){pI=true;return;}string[]gt=gp.Split(' ') 
;var gu="";t3 gv=new t3(gq,gr,gs);if(gt.Length==2){if(!pD.TryGetValue(gt[1],out gu))gu=gt[1];}var gw=gt[0];if(pD.TryGetValue(gw, 
out gv.pY)){pG.q2(gv.pY,gv);return;}sU.nA(ref gw,ref gu);if(gu==""){gv.pX=gw.ToLower();pF.q2(gv.pX,gv);return;}gv.pX=gw;gv.pY=gu; 
pH.q2(gw.ToLower()+' '+gu.ToLower(),gv);}public t3 pM(string gx,string gy,string gz){t3 gA;gx=gx.ToLower();gA=pH.q4(gx);if(gA!= 
null)return gA;gy=gy.ToLower();gA=pF.q4(gy);if(gA!=null)return gA;gz=gz.ToLower();gA=pG.q4(gz);if(gA!=null)return gA;return null;} 
public bool pN(string gB,string gC,string gD){t3 gE;var gF=false;gE=pG.q4(gD.ToLower());if(gE!=null){if(gE.pZ)return true;gF=true; 
}gE=pF.q4(gC.ToLower());if(gE!=null){if(gE.pZ)return true;gF=true;}gE=pH.q4(gB.ToLower());if(gE!=null){if(gE.pZ)return true;gF= 
true;}return!(pI||gF);}public t3 pO(string gG,string gH,string gI){t3 gJ=new t3();gG=gG.ToLower();t3 gK=pM(gG,gH.ToLower(),gI. 
ToLower());if(gK!=null){gJ.pV=gK.pV;gJ.pW=gK.pW;}gJ.pX=gH;gJ.pY=gI;pJ.q2(gG,gJ);return gJ;}public t3 pP(string gL,string gM,string 
gN){t3 gO=pJ.q4(gL.ToLower());if(gO==null)gO=pO(gL,gM,gN);return gO;}int pQ=0;List<t3>pR;public List<t3>pS(string gP,bool gQ){if(! 
gQ){pR=new List<t3>();pQ=0;}for(;pQ<pJ.q3();pQ++){if(!pE.sa(5))return null;t3 gR=pJ.q5(pQ);if(pN((gR.pX+' '+gR.pY).ToLower(),gR.pX 
,gR.pY))continue;if(gR.pY==gP)pR.Add(gR);}return pR;}int pT=0;public bool pU(bool gS){if(!gS){pT=0;}for(;pT<MMItems.pu.Count;pT++) 
{if(!pE.sa(10))return false;t1 gT=MMItems.ps[MMItems.pu[pT]];if(!gT.pC)continue;var gU=gT.px+' '+gT.py;if(pN(gU,gT.px,gT.py)) 
continue;t3 gV=pP(gU,gT.px,gT.py);if(gV.pW==-1)gV.pW=gT.pz;}return true;}}public class t3 {public int pV;public int pW;public 
string pX="";public string pY="";public bool pZ;public double p_;public t3(bool gW=false,int gX=1,int gY=-1){pV=gX;pZ=gW;pW=gY;}} 
public class t4 {Dictionary<string,t3>q0=new Dictionary<string,t3>();List<string>q1=new List<string>();public void q2(string gZ,t3 
g_){if(!q0.ContainsKey(gZ)){q1.Add(gZ);q0.Add(gZ,g_);}}public int q3(){return q0.Count;}public t3 q4(string h0){if(q0.ContainsKey( 
h0))return q0[h0];return null;}public t3 q5(int h1){return q0[q1[h1]];}public void q6(){q1.Clear();q0.Clear();}public void q7(){q1 
.Sort();}}public class t5 {tb q8;public MyDefinitionId q9=new MyDefinitionId(typeof(VRage.Game.ObjectBuilders.Definitions. 
MyObjectBuilder_GasProperties),"Electricity");public MyDefinitionId qa=new MyDefinitionId(typeof(VRage.Game.ObjectBuilders. 
Definitions.MyObjectBuilder_GasProperties),"Oxygen");public MyDefinitionId qb=new MyDefinitionId(typeof(VRage.Game.ObjectBuilders. 
Definitions.MyObjectBuilder_GasProperties),"Hydrogen");public t5(tb h2){q8=h2;}int qc=0;public bool qd(List<IMyTerminalBlock>h3, 
ref double h4,ref double h5,ref double h6,ref double h7,ref double h8,ref double h9,bool ha){if(!ha)qc=0;MyResourceSinkComponent 
hb;MyResourceSourceComponent hc;for(;qc<h3.Count;qc++){if(!q8.sa(8))return false;if(h3[qc].Components.TryGet< 
MyResourceSinkComponent>(out hb)){h4+=hb.CurrentInputByType(q9);h5+=hb.MaxRequiredInputByType(q9);}if(h3[qc].Components.TryGet< 
MyResourceSourceComponent>(out hc)){h6+=hc.CurrentOutputByType(q9);h7+=hc.MaxOutputByType(q9);}h8+=(h3[qc]as IMyBatteryBlock). 
CurrentStoredPower;h9+=(h3[qc]as IMyBatteryBlock).MaxStoredPower;}return true;}int qe=0;public bool qf(List<IMyTerminalBlock>hd, 
MyDefinitionId he,ref double hf,ref double hg,ref double hh,ref double hi,bool hj){if(!hj)qe=0;MyResourceSinkComponent hk; 
MyResourceSourceComponent hl;for(;qe<hd.Count;qe++){if(!q8.sa(6))return false;if(hd[qe].Components.TryGet<MyResourceSinkComponent> 
(out hk)){hf+=hk.CurrentInputByType(he);hg+=hk.MaxRequiredInputByType(he);}if(hd[qe].Components.TryGet<MyResourceSourceComponent>( 
out hl)){hh+=hl.CurrentOutputByType(he);hi+=hl.MaxOutputByType(he);}}return true;}int qg=0;public bool qh(List<IMyTerminalBlock>hm 
,string hn,ref double ho,ref double hp,bool hq){hn=hn.ToLower();if(!hq){qg=0;hp=0;ho=0;}MyResourceSinkComponent hr;for(;qg<hm. 
Count;qg++){if(!q8.sa(30))return false;IMyOxygenTank hs=hm[qg]as IMyOxygenTank;if(hs==null)continue;double ht=0;if(hs.Components. 
TryGet<MyResourceSinkComponent>(out hr)){ListReader<MyDefinitionId>hu=hr.AcceptedResources;int hv=0;for(;hv<hu.Count;hv++){if(hu[ 
hv].SubtypeId.ToString().ToLower()==hn){ht=100;hp+=ht;ho+=ht*hs.GetOxygenLevel();break;}}}}return true;}}public class t6 {t8 qi= 
null;public t7 qj=new t7();public t9 qk=null;public IMyTextPanel ql=null;public int qm=0;public string qn="";public string qo=""; 
public bool qp=true;public t6(t8 hw,string hx){qi=hw;qo=hx;}public bool qq(){return qk.rq.Count>qk.rm||qk.rn!=0;}public void qr( 
float hy){for(int hz=0;hz<qj.qz();hz++)qj.qB(hz).SetValueFloat("FontSize",hy);}public void qs(){qj.qD();ql=qj.qB(0);int hA=ql. 
CustomName.IndexOf("!MARGIN:");if(hA<0||hA+8>=ql.CustomName.Length){qm=1;qn=" ";}else{var hB=ql.CustomName.Substring(hA+8);int hC= 
hB.IndexOf(" ");if(hC>=0)hB=hB.Substring(0,hC);if(!int.TryParse(hB,out qm))qm=1;qn=new String(' ',qm);}if(ql.CustomName.Contains( 
"!NOSCROLL"))qp=false;else qp=true;}public bool qt(){return(ql.BlockDefinition.SubtypeId.Contains("Wide")||ql. 
DefinitionDisplayNameText=="Computer Monitor");}public void qu(){if(qk==null||ql==null)return;float hD=ql.GetValueFloat("FontSize" 
);for(int hE=0;hE<qj.qz();hE++){IMyTextPanel hF=qj.qB(hE);if(hE>0)hF.SetValueFloat("FontSize",hD);hF.WritePublicText(qk.rC(hE));if 
(qi.qM)hF.ShowTextureOnScreen();hF.ShowPublicTextOnScreen();}}}public class t7 {Dictionary<string,IMyTextPanel>qv=new Dictionary< 
string,IMyTextPanel>();List<string>qw=new List<string>();public void qx(string hG,IMyTextPanel hH){if(!qw.Contains(hG)){qw.Add(hG) 
;qv.Add(hG,hH);}}public void qy(string hI){if(qw.Contains(hI)){qw.Remove(hI);qv.Remove(hI);}}public int qz(){return qv.Count;} 
public IMyTextPanel qA(string hJ){if(qw.Contains(hJ))return qv[hJ];return null;}public IMyTextPanel qB(int hK){return qv[qw[hK]];} 
public void qC(){qw.Clear();qv.Clear();}public void qD(){qw.Sort();}}public class t8 {public const float qG=512/0.7783784f;public 
string qI="T:[LCD]";public int qJ=1;public bool qK=true;public List<string>qL=null;public bool qM=true;public bool qN=false;public 
float qO=1.0f;public float qP=1.0f;public float qQ;float qR;float qS;float qT;int qU=0;string qV="";tb qW;public MyGridProgram qX; 
public t5 qY;public s_ qZ;public sY q_;public IMyGridTerminalSystem r0{get{return qX.GridTerminalSystem;}}public 
IMyProgrammableBlock r1{get{return qX.Me;}}public Action<string>r2{get{return qX.Echo;}}public t8(MyGridProgram hL,bool hM,tb hN){ 
qW=hN;qN=hM;qX=hL;qY=new t5(hN);qZ=new s_(hN);qZ.ph();tc.sj();qR=tc.sg(' ');qS=tc.sg('\'');qT=tc.sg('[');qQ=tc.sh(" 100.0%");q_= 
new sY(qW);qW.s2(q_,0);}Dictionary<IMyTextPanel,t9>r3=new Dictionary<IMyTextPanel,t9>();public t9 r4=null;public t9 r5(t9 hO,t6 hP 
){hP.qs();IMyTextPanel hQ=hP.ql;if(hO==null)hO=new t9(this,hQ.GetValueFloat("FontSize"));else hO.rt(hQ.GetValueFloat("FontSize")); 
hO.ru(hP.qj.qz());hO.rl=(hP.qt()?2.0f:1.0f)*qO/hO.rk;qV=hP.qn;qU=hP.qm;r4=hO;return hO;}public void r6(t6 hR){r4=hR.qk;}public 
void r7(string hS){if(r4.rs<=0)hS=qV+hS;r4.rz(hS);}public void r8(string hT){r4.ry(hT,qV);}public void r9(List<string>hU){r4.rw(hU 
);}public void Add(string hV){if(r4.rs<=0)hV=qV+hV;r4.rv(hV);r4.rs+=tc.sh(hV);}public void ra(string hW,float hX=1.0f,float hY=0f) 
{rb(hW,hX,hY);r7("");}public void rb(string hZ,float h_=1.0f,float i0=0f){float i1=tc.sh(hZ);float i2=h_*qG*r4.rl-r4.rs-i0;if(qU>0 
)i2-=qR*qU;if(i2<i1){r4.rv(hZ);r4.rs+=i1;return;}i2-=i1;int i3=(int)Math.Floor(i2/qR);float i4=i3*qR;r4.rv(new String(' ',i3)+hZ); 
r4.rs+=i4+i1;}public void rc(string i5){rd(i5);r7("");}public void rd(string i6){float i7=tc.sh(i6);float i8=qG/2*r4.rl-r4.rs;if( 
i8<i7/2){r4.rv(i6);r4.rs+=i7;return;}i8-=i7/2;int i9=(int)Math.Round(i8/qR,MidpointRounding.AwayFromZero);float ia=i9*qR;r4.rv(new 
String(' ',i9)+i6);r4.rs+=ia+i7;}public void re(double ib,float ic=1.0f,float id=0f){if(qU>0)id+=qU*qR*((r4.rs<=0)?2:1);float ie= 
qG*ic*r4.rl-r4.rs-id;if(Double.IsNaN(ib))ib=0;int ig=(int)(ie/qS)-2;if(ig<=0)ig=2;int ih=Math.Min((int)(ib*ig)/100,ig);if(ih<0)ih= 
0;r4.rz((r4.rs<=0?qV:"")+"["+new String('|',ih)+new String('\'',ig-ih)+"]");}public void rf(double ii,float ij=1.0f,float ik=0f){ 
if(qU>0)ik+=qU*qR*((r4.rs<=0)?2:1);float il=qG*ij*r4.rl-r4.rs-ik;if(Double.IsNaN(ii))ii=0;int im=(int)(il/qS)-2;if(im<=0)im=2;int 
io=Math.Min((int)(ii*im)/100,im);if(io<0)io=0;r4.rv((r4.rs<=0?qV:"")+"["+new String('|',io)+new String('\'',im-io)+"]");r4.rs+=(r4 
.rs<=0?qU*qR:0)+qS*im+2*qT;}public void rg(){r4.rA();}public void rh(t6 ip){ip.qu();if(ip.qp)r4.rG();}public void ri(string iq, 
string ir){IMyTextPanel it=qX.GridTerminalSystem.GetBlockWithName(iq)as IMyTextPanel;if(it==null)return;it.WritePublicText(ir+"\n" 
,true);}}public class t9 {t8 rj=null;public float rk=1.0f;public float rl=1.0f;public int rm=17;public int rn=0;int ro=1;int rp=1; 
public List<string>rq=new List<string>();public int rr=0;public float rs=0;public t9(t8 iu,float iv=1.0f){rj=iu;rt(iv);rq.Add(""); 
}public void rt(float iw){rk=iw;rm=(int)Math.Floor(512/0.7783784f*rj.qP*rp/rk/37);}public void ru(int ix){rp=ix;rm=(int)Math.Floor 
(512/0.7783784f*rj.qP*rp/rk/37);}public void rv(string iy){rq[rr]+=iy;}public void rw(List<string>iz){if(rq[rr]=="")rq.RemoveAt(rr 
);else rr++;rq.AddList(iz);rr+=iz.Count;rq.Add("");rs=0;}public List<string>rx(){if(rq[rr]=="")return rq.GetRange(0,rr);else  
return rq;}public void ry(string iA,string iB=""){string[]iC=iA.Split('\n');for(int iD=0;iD<iC.Length;iD++)rz(iB+iC[iD]);}public 
void rz(string iE){rq[rr]+=iE;rq.Add("");rr++;rs=0;}public void rA(){rq.Clear();rq.Add("");rs=0;rr=0;}public string rB(){return 
String.Join("\n",rq);}public string rC(int iF=0){if(rq.Count<rm/rp){if(iF!=0)return "";rn=0;ro=1;return rB();}int iG=rn+iF*(rm/rp) 
;if(iG>rq.Count)iG=rq.Count;List<string>iH=rq.GetRange(iG,Math.Min(rq.Count-iG,rm/rp));return String.Join("\n",iH);}public bool rD 
(int iI=-1){if(iI<=0)iI=rj.qJ;if(rn-iI<=0){rn=0;return true;}rn-=iI;return false;}public bool rE(int iJ=-1){if(iJ<=0)iJ=rj.qJ;int 
iK=rq.Count-1;if(rn+iJ+rm>=iK){rn=Math.Max(iK-rm,0);return true;}rn+=iJ;return false;}public int rF=0;public void rG(){if(rF>0){rF 
--;return;}if(rq.Count-1<=rm){rn=0;ro=1;return;}if(ro>0){if(rE()){ro=-1;rF=2;}}else{if(rD()){ro=1;rF=2;}}}}public class ta {public 
string rH="MMTask";public double rI=0;public double rJ=0;public double rK=0;public double rL=-1;public bool rM=false;public bool 
rN=false;double rO=0;protected tb rP;public void rQ(tb iL){rP=iL;}protected bool rR(double iM){rO=Math.Max(iM,0.0001);return true; 
}public bool rS(){if(rJ>0){rK=rP.rX-rJ;rP.s4((rN?"Running":"Resuming")+" task: "+rH);rN=Run(!rN);}else{rK=0;rP.s4("Init task: "+rH 
);Init();rP.s4("Running..");rN=Run(false);if(!rN)rJ=0.001;}if(rN){rJ=rP.rX;if((rL>=0||rO>0)&&rM)rP.s2(this,(rO>0?rO:rL));else{rM= 
false;rJ=0;}}else{if(rM)rP.s2(this,0,true);}rP.s4("Task "+(rN?"":"NOT ")+"finished. "+(rM?(rO>0?"Postponed by "+rO.ToString("F1")+ 
"s":"Scheduled after "+rL.ToString("F1")+"s"):"Stopped."));rO=0;return rN;}public void rT(){rP.s3(this);End();rM=false;rN=false;rJ 
=0;}public virtual void Init(){}public virtual bool Run(bool iN){return true;}public virtual void End(){}}public class tb {public 
double rX{get{return rZ;}}int rY=1000;double rZ=0;List<ta>r_=new List<ta>(100);public MyGridProgram s0;bool s1=true;public tb( 
MyGridProgram iO,bool iP=true){s0=iO;s1=iP;}public void s2(ta iQ,double iR,bool iS=false){s4("Scheduling task: "+iQ.rH+" after "+ 
iR.ToString("F2"));iQ.rM=true;iQ.rQ(this);if(iS){iQ.rI=rX;r_.Insert(0,iQ);return;}if(iR<=0)iR=0.001;iQ.rI=rX+iR;for(int iT=0;iT<r_ 
.Count;iT++){if(r_[iT].rI>iQ.rI){r_.Insert(iT,iQ);return;}if(iQ.rI-r_[iT].rI<0.05)iQ.rI=r_[iT].rI+0.05;}r_.Add(iQ);}public void s3 
(ta iU){if(r_.Contains(iU)){r_.Remove(iU);iU.rM=false;}}public void s4(string iV){if(s1)s0.Echo(iV);}public void s5(){double iW=s0 
.Runtime.TimeSinceLastRun.TotalSeconds;rZ+=iW;s4("Total time: "+rZ.ToString("F1")+" Time Step: "+iW.ToString("F2"));rY=(int)Math. 
Min((iW*60)*1000,20000-1000);s4("Total tasks: "+r_.Count+" InstrPerRun: "+rY);while(r_.Count>=1){ta iX=r_[0];if((rY-s0.Runtime. 
CurrentInstructionCount<=0)||(s0.Runtime.MaxMethodCallCount-s0.Runtime.CurrentMethodCallCount<=100))break;if(iX.rI>rZ)break;r_. 
Remove(iX);if(!iX.rS())break;s4("Done. NextExecTime: "+iX.rI.ToString("F1"));s4("Remaining I / M: "+s9().ToString()+" / "+(s0. 
Runtime.MaxMethodCallCount-s0.Runtime.CurrentMethodCallCount).ToString());}}int s6=0;StringBuilder s7=new StringBuilder();public 
void s8(){double iY=s0.Runtime.LastRunTimeMs*1000;if(s6==5000){IMyTextPanel iZ=s0.GridTerminalSystem.GetBlockWithName( 
"AUTOLCD Profiler")as IMyTextPanel;if(iZ==null)return;iZ.WritePublicText(s7.ToString());s6++;return;}s7.Append(s6).Append(";"). 
AppendLine(iY.ToString("F2"));s6++;}public int s9(){return(20000-s0.Runtime.CurrentInstructionCount);}public bool sa(int i_){ 
return(((20000-s0.Runtime.CurrentInstructionCount)>=i_)&&(s0.Runtime.MaxMethodCallCount-s0.Runtime.CurrentMethodCallCount-100>=i_) 
);}public void sb(){s4("Remaining I / M: "+s9().ToString()+" / "+(s0.Runtime.MaxMethodCallCount-s0.Runtime.CurrentMethodCallCount) 
.ToString());}}public static class tc {static Dictionary<char,float>sc=new Dictionary<char,float>();static float sd=1f;static 
float se=10f;public static float sf=-1f;static void AddCharsSize(string j0,float j1){j1+=1;for(int j2=0;j2<j0.Length;j2++){if(j1> 
sd)sd=j1;sc.Add(j0[j2],j1);}}public static float sg(char j3){float j4;if(!sc.TryGetValue(j3,out j4))return sd;return j4;}public 
static float sh(string j5){if(sf>0)return j5.Length*sd;float j6=0;for(int j7=0;j7<j5.Length;j7++)j6+=sg(j5[j7]);return j6;}public 
static string si(string j8,float j9){if(j9/sd>=j8.Length)return j8;float ja=sh(j8);if(ja<=j9)return j8;float jb=ja/j8.Length;j9-= 
se;int jc=(int)(j9/jb);j8=j8.Substring(0,jc);ja=sh(j8);while(ja>j9){jc--;ja-=sg(j8[jc]);j8.Remove(jc);}return j8+"..";}public 
static void sj(){if(sc.Count>0)return;if(sf>0){sd=sf+1;se=2*sd;return;} 
// Font characters width (font "aw" values here) 
AddCharsSize("3FKTabdeghknopqsuy£µÝàáâãäåèéêëðñòóôõöøùúûüýþÿāăąďđēĕėęěĝğġģĥħĶķńņňŉōŏőśŝşšŢŤŦũūŭůűųŶŷŸșȚЎЗКЛбдекруцяёђћўџ", 17f); 
AddCharsSize("#0245689CXZ¤¥ÇßĆĈĊČŹŻŽƒЁЌАБВДИЙПРСТУХЬ€", 19f); 
AddCharsSize("$&GHPUVY§ÙÚÛÜÞĀĜĞĠĢĤĦŨŪŬŮŰŲОФЦЪЯжы†‡", 20f); 
AddCharsSize("ABDNOQRSÀÁÂÃÄÅÐÑÒÓÔÕÖØĂĄĎĐŃŅŇŌŎŐŔŖŘŚŜŞŠȘЅЊЖф□", 21f); 
AddCharsSize("(),.1:;[]ft{}·ţťŧț", 9f); 
AddCharsSize("+<=>E^~¬±¶ÈÉÊË×÷ĒĔĖĘĚЄЏЕНЭ−", 18f); 
AddCharsSize(" !I`ijl ¡¨¯´¸ÌÍÎÏìíîïĨĩĪīĮįİıĵĺļľłˆˇ˘˙˚˛˜˝ІЇії‹›∙", 8f); 
AddCharsSize("7?Jcz¢¿çćĉċčĴźżžЃЈЧавийнопсъьѓѕќ", 16f); 
AddCharsSize("L_vx«»ĹĻĽĿŁГгзлхчҐ–•", 15f); 
AddCharsSize("\"-rª­ºŀŕŗř", 10f); 
AddCharsSize("mw¼ŵЮщ", 27f); 
AddCharsSize("MМШ", 26f); 
AddCharsSize("WÆŒŴ—…‰", 31f); 
AddCharsSize("'|¦ˉ‘’‚", 6f); 
AddCharsSize("*²³¹", 11f); 
AddCharsSize("\\°“”„", 12f); 
AddCharsSize("/ĳтэє", 14f); 
AddCharsSize("%ĲЫ", 24f); 
AddCharsSize("@©®мшњ", 25f); 
AddCharsSize("\n", 0f); 
AddCharsSize("¾æœЉ", 28f); 
AddCharsSize("½Щ", 29f); 
AddCharsSize("ј", 7f); 
AddCharsSize("љ", 22f); 
AddCharsSize("ґ", 13f); 
AddCharsSize("ю", 23f); 
AddCharsSize("™", 30f); 
AddCharsSize("", 34f); 
AddCharsSize("", 32f); 
AddCharsSize("", 41f); 
// End of font characters width 
 se=sg('.')*2;}} 
 
 
public static class M 
{ 
public static readonly Dictionary<string, string> T = new Dictionary<string, string> 
{ 
// TRANSLATION STRINGS 
// msg id, text 
{ "AC1", "Ускорение:" }, 
{ "A1", "ПУСТО" }, // amount 
{ "ALT1", "Абсолютная высота:"}, // NEW 
{ "ALT2", "Высота Полета:"}, // NEW 
{ "B1", "ЗАГРУЗКА..." }, 
{ "C1", "всего:" }, 
{ "C2", "Объем Груза:" }, 
{ "C3", "Invalid countdown format, use:" }, 
{ "C4", "EXPIRED" }, 
{ "C5", "дней" }, 
{ "D1", "Не введенно Имя" }, 
{ "D2", "Блок не Найден." }, 
{ "D3", "Поврежденых блоков нет." }, 
{ "DTU", "Неправильный GPS формат" }, // NEW 
{ "GA", "Искусс.Г."}, // NEW (don't use more than 5 characters) 
{ "GN", "Природ.Г."}, // NEW (don't use more than 5 characters) 
{ "GT", "Общая.Г."}, // NEW (don't use more than 5 characters) 
{ "G1", "Общая Гравитация:"}, // NEW 
{ "G2", "Природная Гравитация:"}, // NEW 
{ "G3", "Искусственная Гравитация:"}, // NEW 
{ "H1", "Write commands to Custom Data of this panel." }, // UPDATED 
// inventory 
{ "I1", "Руды" }, 
{ "I2", "в наличии" }, 
{ "I3", "Руда" }, 
{ "I4", "Слитки" }, 
{ "I5", "Компоненты" }, 
{ "I6", "Балоны" }, 
{ "I7", "Патроны" }, 
{ "I8", "Инструменты и Оружие" }, 
{ "M1", "Масса Груза:" }, 
// oxygen 
{ "O1", "Разгерметизация" }, 
{ "O2", "Кислородная ферма" }, 
{ "O3", "No oxygen blocks found." }, 
{ "O4", "Кислородный бак" }, 
// position 
{ "P1", "Блок не найден" }, 
{ "P2", "Координаты" }, 
// power 
{ "P3", "Накоплено" }, 
{ "P4", "Выходное" }, 
{ "P5", "Входное" }, 
{ "P6", "Генераторов энергии НЕТ!" }, 
{ "P7", "Батареи" }, 
{ "P8", "Общее Выходное" }, 
{ "P9", "Реакторы" }, 
{ "P10", "Сол.Панели" }, 
{ "P11", "Мощьность" }, 
{ "PT1", "Время работы:" }, 
{ "PT2", "Время работы:" }, 
{ "PU1", "Энергопотребление:" }, // NEW 
{ "S1", "Скорость:" }, 
{ "SM1", "Масса Корабля:" }, // NEW 
{ "SM2", "Базавая Масса:" }, // NEW 
{ "SD", "Дистанция Остановки:" }, // NEW 
{ "ST", "Время Остановки:" }, // NEW 
// text 
{ "T1", "Source LCD not found: " }, 
{ "T2", "Missing source LCD name" }, 
{ "T3", "LCD Private Text is empty" }, 
// tanks 
{ "T4", "Ошибка в названии. прим: 'Tanks * Hydrogen'" }, 
{ "T5", "нет {0} такого бака." }, // {0} is tank type 
{ "T6", "Бак" }, 
{ "UC", "Неизвестная команда" }, 
// occupied & dampeners 
{ "SC1", "не найден блок управления." }, 
{ "SCD", "Dampeners: " }, 
{ "SCO", "Occupied: " }, 
// working 
{ "W1", "Выключено" }, 
{ "W2", "Работает" }, 
{ "W3", "Ожидание" }, 
{ "W4", "Разгерметизация" }, 
{ "W5", "Открыто" }, 
{ "W6", "Закрыто" }, 
{ "W7", "Заблокировано" }, 
{ "W8", "Разблокировано" }, 
{ "W9", "Включено" }, 
{ "W10", "Готов" } 
}; 
}
